@@ -25,7 +25,8 @@ data Const = Num Int
 instance Show Const where
     show (Num i) = show i
     show (Ch ch) = show ch
-    show (Bo b) = show b
+    show (Bo False) = "false"
+    show (Bo True) = "true"
     show (Re d) = show d
     show (Str str) = show str
     show (Exc str i) = "exception(" ++ show str ++ ", " ++ show i ++ ")"
@@ -34,16 +35,19 @@ instance Show Const where
 data Name = UN String
 	  | MN (String,Int)
 	  | NS Name Name -- Name in a namespace
+          | OP Op -- operator to overload
 	  | None
    deriving (Eq, Ord)
 
 type LongName = [Name]
 
 instance Show Name where
-    show (UN s) = "_D_" ++ quotename s
-    show (NS s n) = "_ns_"++ show s ++ "_" ++ show n
-    show (MN (s,i)) = "_my_" ++ quotename s ++ show i
-    show None = "_DNAME__"
+    show n = "K" ++ show' n -- dlsym can't cope with underscores in front!
+       where show' (UN s) = "_D_" ++ quotename s
+             show' (NS s n) = "_ns_"++ show' s ++ "_" ++ show' n
+             show' (MN (s,i)) = "_my_" ++ quotename s ++ show i
+             show' (OP op) = "_op_" ++ opName op
+             show' None = "_DNAME__"
 
 quotename [] = ""
 quotename ('_':cs) = "_UN_"++quotename cs
@@ -59,6 +63,7 @@ quotename (c:cs) = c:(quotename cs)
 showuser (UN s) = s
 showuser (MN (s,i)) = "<"++show s ++ show i++">"
 showuser (NS s n) = showuser s ++ "::" ++ showuser n
+showuser (OP op) = "operator" ++ show op
 showuser None = error "Please don't do that"
 
 showlist [] = ""
@@ -70,7 +75,6 @@ data PrimType = Number
 	      | Boolean
 	      | RealNum
 	      | StringType
-	      | File
 	      | Pointer
 	      | Exception
 	      | Void
@@ -82,7 +86,6 @@ instance Show PrimType where
   show Boolean = "Bool"
   show RealNum = "Float"
   show StringType = "String"
-  show File = "File"
   show Pointer = "Ptr"
   show Exception = "%Exception"
   show Void = "Void"
@@ -90,7 +93,11 @@ instance Show PrimType where
 data Type = Prim PrimType
 	  | Fn [Maybe Raw] [Type] Type -- Defaults, arg types, return type
 	  | Array Type
-	  | User Name [Type] -- User type, parametrised over types
+--	  | User Name [Type] -- User type, parametrised over types
+-- Want this instead, so that we can have type variables of kind other than
+-- *:
+          | User Name
+          | TyApp Type [Type]
 --	  | Syn Name -- Type synonym - rapidly expanded out by typechecker
 	  | TyVar Name
 	  | UnknownType -- We don't know what the type is yet.
@@ -103,7 +110,8 @@ instance Eq Type where
     (==) (Prim t1) (Prim t2) = t1==t2
     (==) (Fn _ ts t) (Fn _ ts2 t2) = ts==ts2 && t==t2
     (==) (Array t1) (Array t2) = t1==t2
-    (==) (User n1 t1) (User n2 t2) = n1==n2 && t1==t2
+    (==) (User n1) (User n2) = n1==n2
+    (==) (TyApp n1 t1) (TyApp n2 t2) = n1==n2 && t1==t2
 --    (==) (Syn n1) (Syn n2) = n1==n2
     (==) (TyVar n1) (TyVar n2) = n1==n2
     (==) UnknownType UnknownType = True
@@ -113,7 +121,8 @@ instance Show Type where
     show (Prim t) = show t
     show (Array t) = "[" ++ show t ++ "]"
     show (Fn ns ts t) = show t ++ "(" ++ showlist ts ++ ")"
-    show (User n tys) = showuser n ++ showargs tys
+    show (User n) = showuser n
+    show (TyApp n tys) = show n ++ showargs tys
 	    where showargs [] = ""
 		  showargs (x:xs) = "<" ++ sa' (x:xs) ++ "> "
 		  sa' [] = ""
@@ -131,12 +140,14 @@ instance Show Type where
 data InputType = Program String
 	       | Shebang
 	       | Module
+               | SharedLib -- for the REPL
   deriving Eq
 
 instance Show InputType where
     show (Program str) = str
     show Shebang = "script"
     show Module = "module"
+    show SharedLib = "library"
 
 data ArgType = Var | Copy
   deriving (Show, Eq)
@@ -173,11 +184,14 @@ data Raw = RVar String Int Name
 	 | RUnary String Int UnOp Raw
 	 | RCoerce String Int Type Raw
 	 | RCase String Int Raw [RCaseAlt]
+         | RMatch String Int Raw [MatchAlt]
 	 | RIf String Int Raw Raw Raw
 	 | RIndex String Int Raw Raw
 	 | RField String Int Raw Name
 	 | RArrayInit String Int [Raw]
 	 | RVMPtr String Int
+         | RLength String Int Raw -- Get directly at a string's length
+         | RUnderscore String Int -- match anything pattern
 	 | RNoop String Int
 	 | RMetavar String Int Int
    deriving (Show, Eq)
@@ -191,9 +205,19 @@ data RCatch = RCatch String Int
 
 data RCaseAlt = RAlt String Int Name [Name] Raw
               | RConstAlt String Int Const Raw
+              | RArrayAlt String Int [Name] Raw
               | RDefault String Int Raw
    deriving (Show, Eq)
 
+data MatchAlt = MAlt String Int 
+                     [Raw] -- patterns 
+                     Raw -- result
+   deriving (Show, Eq)
+
+data LCClause = LCGet String Int Name Raw
+              | LCFilter String Int Raw
+   deriving (Show, Eq)
+
 data RAssign = RAName String Int Name
 	     | RAIndex String Int RAssign Raw
 	     | RAField String Int RAssign Name
@@ -206,7 +230,7 @@ data Foreign = ForeignDecl String Int [FOpt] Type
 data Op = Plus | Minus | Times | Divide | Modulo | Power | Equal | NEqual |
 	  OpLT | OpGT | OpLE | OpGE | OpAnd | OpOr | OpXOR | BAnd | BOr |
 	  OpShLeft | OpShRight | OpAndBool | OpOrBool
-   deriving Eq
+   deriving (Eq, Ord)
 
 instance Show Op where
     show Plus = "+"
@@ -231,6 +255,28 @@ instance Show Op where
     show OpAndBool = "&&"
     show OpOrBool = "||"
 
+opName Plus = "plus"
+opName Minus = "minus"
+opName Times = "times"
+opName Divide = "div"
+opName Modulo = "mod"
+opName Power = "pow"
+opName Equal = "eq"
+opName NEqual = "neq"
+opName OpLT = "lt"
+opName OpGT = "gt"
+opName OpLE = "le"
+opName OpGE = "ge"
+opName OpAnd = "and"
+opName OpOr = "or"
+opName OpXOR = "xor"
+opName BAnd = "band" -- FIXME: BAnd seems unnecessary
+opName BOr = "bor" -- FIXME: BOr seems unnecessary
+opName OpShLeft = "shl"
+opName OpShRight = "shr"
+opName OpAndBool = "and"
+opName OpOrBool = "or"
+
 boolops = [Equal, NEqual, OpLT, OpGT, OpLE, OpGE, BAnd, BOr]
 
 data UnOp = Not | Neg
@@ -267,6 +313,7 @@ data Expr n = Global n String Int
             | AssignApp (Assign n) (Expr n)
 	    | Seq (Expr n) (Expr n)
 	    | Apply (Expr n) [Expr n]
+            | ConApply (Expr n) [Expr n]
 	    | Partial (Expr n) [Expr n] Int
 	    | Foreign Type n [(Expr n,Type)]
 	    | While (Expr n) (Expr n)
@@ -301,8 +348,10 @@ data Expr n = Global n String Int
 	    | Field (Expr n) Name Int Int -- name, argument and tag
 	    | ArrayInit [Expr n]
 	    | VMPtr
+            | Length (Expr n)
 	    | Error String
 	    | Noop
+	    | NoInit
 	    | Metavar String Int Int -- Hole for optimisations
 	    | Annotation Annotation (Expr n) -- Annotation expression
   deriving (Show, Eq)
@@ -315,6 +364,7 @@ data Catch n = Catch -- Left, exception name and arguments
 
 data CaseAlt n = Alt Int Int [Expr n] (Expr n) -- tag, total
                | ConstAlt PrimType Const (Expr n)
+               | ArrayAlt [Expr n] (Expr n)
                | Default (Expr n) -- if nothing else matches
    deriving (Show, Eq)
 
@@ -326,12 +376,16 @@ data Annotation = Line String Int -- File, Line number
 
 instance Eq n => Ord (CaseAlt n) where
     compare (Alt x _ _ _) (Alt y _ _ _) = compare x y
+    compare (ArrayAlt xs _) (ArrayAlt ys _) = compare (length xs) (length ys)
     compare (ConstAlt _ x _) (ConstAlt _ y _) = compare x y
+
     compare (Alt _ _ _ _) (Default _) = LT
+    compare (ArrayAlt _ _) (Default _) = LT
     compare (ConstAlt _ _ _) (Default _) = LT
     compare (Default _) (Default _) = EQ
     compare (Default _) (Alt _ _ _ _) = GT
     compare (Default _) (ConstAlt _ _ _) = GT
+    compare _ _ = EQ -- others don't matter
 
 
 data Assign n = AName Int -- Local name
@@ -356,7 +410,7 @@ Public, Pure, Deprecated are user specified, others depend on how
 the function is defined, imported, etc. -}
 data FOpt = Public | Pure | NoArgs | Inline | Export | Generated
 	  | DefaultDef -- can override definition
-          | StartupFn | DeprecatedFn 
+          | StartupFn | DeprecatedFn | Constructor
           | Repeatable -- can repeat definition if type is the same
   deriving (Show, Eq)
 
@@ -485,3 +539,77 @@ instance MonadPlus Result where
     mplus (Success x) _ = (Success x)
     mplus (Failure _ _ _) y = y
 
+-- Substitute a term for a variable in a raw term
+
+rawSubst :: Name -> Raw -> Raw -> Raw
+rawSubst n t (RVar _ _ x) | n == x = t
+rawSubst n t (RQVar _ _ x) | n == x = t
+rawSubst n t (RClosure f l ts r) = RClosure f l ts (rawSubst n t r)
+rawSubst n t (RBind f l nm ty r1 r2) 
+   = RBind f l nm ty (rawSubst n t r1) (rawSubst n t r2)
+rawSubst n t (RDeclare f l nb ty r)
+   = RDeclare f l nb ty (rawSubst n t r)
+rawSubst n t (RReturn f l v) = RReturn f l (rawSubst n t v)
+rawSubst n t (RAssign f l a r) = RAssign f l (rawASubst n t a)
+                                             (rawSubst n t r)
+rawSubst n t (RAssignOp f l op a r) = RAssignOp f l op (rawASubst n t a)
+                                                       (rawSubst n t r)
+rawSubst n t (RSeq f l r1 r2) 
+   = RSeq f l (rawSubst n t r1) (rawSubst n t r2)
+rawSubst n t (RApply f l fn args)
+   = RApply f l (rawSubst n t fn) (map (rawSubst n t) args)
+rawSubst n t (RPartial f l fn args)
+   = RPartial f l (rawSubst n t fn) (map (rawSubst n t) args)
+rawSubst n t (RForeign f l ty nm args)
+   = RForeign f l ty nm (map (rawSubst n t) args)
+rawSubst n t (RWhile f l r1 r2) 
+   = RWhile f l (rawSubst n t r1) (rawSubst n t r2)
+rawSubst n t (RDoWhile f l r1 r2) 
+   = RDoWhile f l (rawSubst n t r1) (rawSubst n t r2)
+rawSubst n t (RFor f l ra nm r1 r2)
+   = RFor f l (rawASubst n t ra) nm (rawSubst n t r1) (rawSubst n t r2)
+rawSubst n t (RTryCatch f l r1 r2 nm r3)
+   = RTryCatch f l (rawSubst n t r1) (rawSubst n t r2) nm (rawSubst n t r3)
+rawSubst n t (RNewTryCatch f l r rcs)
+   = RNewTryCatch f l (rawSubst n t r) (map (rawCSubst n t) rcs)
+rawSubst n t (RThrow f l v) = RThrow f l (rawSubst n t v)
+rawSubst n t (RExcept f l r1 r2) 
+   = RExcept f l (rawSubst n t r1) (rawSubst n t r2)
+rawSubst n t (RPrint f l v) = RPrint f l (rawSubst n t v)
+rawSubst n t (RInfix f l op r1 r2) 
+   = RInfix f l op (rawSubst n t r1) (rawSubst n t r2)
+rawSubst n t (RUnary f l op r1) 
+   = RUnary f l op (rawSubst n t r1)
+rawSubst n t (RCoerce f l ty r1) 
+   = RCoerce f l ty (rawSubst n t r1)
+rawSubst n t (RMatch f l sc alts) 
+   = RMatch f l (rawSubst n t sc) (map (rawMSubst n t) alts)
+rawSubst n t (RCase f l sc alts) 
+   = RCase f l (rawSubst n t sc) (map (rawCaseSubst n t) alts)
+rawSubst n t (RIf f l r1 r2 r3)
+   = RIf f l (rawSubst n t r1) (rawSubst n t r2) (rawSubst n t r3)
+rawSubst n t (RIndex f l r1 r2)
+   = RIndex f l (rawSubst n t r1) (rawSubst n t r2)
+rawSubst n t (RField f l r1 nm)
+   = RField f l (rawSubst n t r1) nm
+rawSubst n t (RArrayInit f l rs)
+   = RArrayInit f l (map (rawSubst n t) rs)
+rawSubst n t (RLength f l r)
+   = RLength f l (rawSubst n t r)
+rawSubst _ _ r = r
+
+rawCSubst n t (RCatch f l c h) = RCatch f l c (rawSubst n t h)
+
+rawMSubst n t (MAlt f l pats res) = MAlt f l pats (rawSubst n t res)
+
+rawCaseSubst n t (RAlt f l nm ns res) = RAlt f l nm ns (rawSubst n t res)
+rawCaseSubst n t (RConstAlt f l c res) = RConstAlt f l c (rawSubst n t res)
+rawCaseSubst n t (RArrayAlt f l ns res) = RArrayAlt f l ns (rawSubst n t res)
+rawCaseSubst n t (RDefault f l res) = RDefault f l (rawSubst n t res)
+
+rawASubst n t assign@(RAName f l nm) = assign
+rawASubst n t (RAIndex f l a r) = RAIndex f l (rawASubst n t a)
+                                              (rawSubst n t r)
+rawASubst n t (RAField f l a nm) = RAField f l (rawASubst n t a) nm
+
+
