@@ -40,28 +40,35 @@ writeC mod domain lds ctxt xs out copts
 	 hPutStr out str
 	 hClose out
 
+-- Write out the whole program as a C file. Write declarations as C names,
+-- so that they aren't needlessly decorated more in C++ style. This'll make
+-- it possible for us to know the name and load functions dynamically in
+-- the Read-Eval-Print Loop.
 writeout :: String -> String -> [Output] -> String
 writeout header mprog xs = header ++
-		writedecls xs ++
-		writeout' xs ++ mprog
+                           externC (writedecls xs) ++
+		           writeout' xs ++ mprog
 
 writeout' [] = ""
 writeout' ((RawOutput str):xs) = str ++ "\n" ++ writeout' xs
 writeout' ((ExternOutput (f, ty)):xs) 
-    = "void " ++ (show f) ++ mangling ty ++ "(VMState* vm);\n" 
+    = externC $ "void " ++ (show f) ++ mangling ty ++ "(VMState* vm);\n" 
 				  ++ writeout' xs
 writeout' ((ExcOutput nm defhere):xs)
-    = (if defhere then "" else "extern ") ++
-         "char* " ++ show nm ++ 
-      (if defhere then " = \"" ++ showuser nm ++"\"" else "") ++
-         ";\n" ++ writeout' xs
+    = externC $
+         (if defhere then "" else "extern ") ++
+            "const char* " ++ show nm ++ 
+         (if defhere then " = \"" ++ showuser nm ++"\"" else "") ++
+            ";\n" ++ writeout' xs
 writeout' ((FNOutput (f, ty, pop, args, def)):rest) 
     = "void " ++ (show f) ++ mangling ty ++ "(VMState* vm){\n" ++ pop ++ "\n" ++ def ++ "}\n\n"
       ++ writeout' rest 
 writeout' ((GlobOutput n):rest)
-    = "DECLGLOBAL("++show n++");\n"
+    = externC $ "DECLGLOBAL("++show n++");\n"
       ++ writeout' rest
 
+externC x = "\nextern \"C\" {\n" ++ x ++ "\n}\n"
+
 writedecls [] = ""
 writedecls ((FNOutput (f, ty, _, _, _)):rest) 
     = "void " ++ (show f) ++ mangling ty ++ "(VMState* vm);\n"
@@ -78,6 +85,7 @@ getmain mod lds ctxt Shebang copts =
       str' <- replaceDefs mod ctxt ["__start","__panic"] str copts
       return str'
 getmain mod lds ctxt Module copts = return ""
+getmain mod lds ctxt SharedLib copts = return ""
 {-
 getmain mod lds ctxt Webapp = 
    do str <- findFile lds "startup.vcc"
@@ -185,11 +193,13 @@ instr (DECLARE v) = "DECLARE("++show (fst v)++")"
 instr (DECLAREARG v) = "DECLAREARG("++show (fst v)++")"
 instr (DECLAREQUICK v) = "DECLAREQUICK("++show (fst v)++")"
 instr (HEAPVAL v) = "HEAPVAL("++ show (fst v) ++ ")"
-instr (USETMP v) = "DECLAREARG("++show (fst v)++")"
+--instr (USETMP v) = "DECLAREARG("++show (fst v)++")"
 instr (TMPINT i) = "TMPINT(t"++show i ++")"
 instr (TMPREAL i) = "TMPREAL(t"++show i ++")"
 instr (SET var idx val) = "SET("++show (fst var)++","++show idx ++ ","++show (fst val)++")"
 instr TOINDEX = "TOINDEX"
+instr SETRV = "SETRV"
+instr (SETRVREAL r) = "SETRVREAL(t"++show r++")"
 instr SETTOP = "SETTOP"
 instr ADDTOP = "ADDTOP"
 instr SUBTOP = "SUBTOP"
@@ -217,6 +227,8 @@ instr (CLOSURETOP i) = "CLOSURETOP("++show i++")"
 instr (FOREIGNCALL n lib ty args) = mkfcall n lib ty args 
 instr (MKCON t i) = "MKCON("++show t ++ "," ++ show i ++ ")"
 instr (MKCONZERO t) = "MKCONZERO("++show t ++ ")"
+instr (MKCONRV t i) = "MKCONRV("++show t ++ "," ++ show i ++ ")"
+instr (MKCONZERORV t) = "MKCONZERORV("++show t ++ ")"
 instr MKEXCEPT = "MKEXCEPT"
 instr (MKNEWEXCEPT n a) = "MKNEWEXCEPT(" ++ show n ++ ", " ++ show a++")"
 instr EQEXCEPT = "EQEXCEPT"
@@ -234,6 +246,7 @@ instr (GETVAL v) = "GETVAL(t"++show v++")"
 instr (GETRVAL v) = "GETRVAL(t"++show v++")"
 instr GETINDEX = "GETINDEX"
 instr (PROJARG a t) = "PROJARG("++ show a ++ "," ++ show t ++ ")"
+instr (SETPROJARG s a t d) = "SETPROJARG("++pushitem s++","++ show a ++ "," ++ show t ++","++pushitem d ++ ")"
 instr (EXPROJARG a t) = "EXPROJARG("++ show (fst a) ++ "," ++ show t ++ ")"
 instr (INFIX t Divide x y) = "INTDIV(" ++ tmp t ++ "," ++ 
 			 tmp x ++ "," ++ tmp y ++ ")"
@@ -290,10 +303,15 @@ instr (PUSH4 x y z w) = "PUSH4("++pushitem x ++ "," ++ pushitem y ++ "," ++
 		                pushitem z ++ "," ++ pushitem w ++ ")"
 instr (STACKINT var v) = "STACKINT("++show (fst var)++","++show v++")";
 instr (STACKTMP var v) = "STACKINT("++show (fst var)++",t"++show v++")";
+instr (STACKREAL var (RVAL v)) = "STACKREAL("++show (fst var)++","++show v++")";
+instr (STACKREAL var (REAL t)) = "STACKREAL("++show (fst var)++",t"++show t++")";
+instr (STACKSTR var (STR v)) = "STACKSTR("++show (fst var)++",L"++show v++")";
 instr (TMPSETTOP t) = "TMPSETTOP(t"++show t++")"
 instr (PUSHSETTOP x) = "PUSHSETTOP("++pushitem x++")"
+instr (SETLOOKUP i v) = "SETLOOKUP("++show i ++", " ++ show (fst v)++")"
 instr (PUSHGETVAL x t) = "PUSHGETVAL("++pushitem x++",t"++show t++")"
 instr (PUSHGETRVAL x t) = "PUSHGETRVAL("++pushitem x++",t"++show t++")"
+instr (INTGETRVAL t) = "INTGETRVAL(t"++show t++")"
 instr (PUSHGETINDEX x) = "PUSHGETINDEX("++pushitem x++")"
 
 instr (PUSHTOINDEX x) = "PUSHTOINDEX("++pushitem x++")"
@@ -331,11 +349,12 @@ instr CHR2STR = "CHR2STR"
 instr INT2REAL = "INT2REAL"
 instr REAL2INT = "REAL2INT"
 instr VMPTR = "VMPTR"
+instr STRLENGTH = "STRLENGTH"
 {- CIM 12/7/05 changed to KERROR to avoid clashes with MinGW -}
 instr ERROR = "KERROR"
 instr (LINENO f l) = "LINENO(L\""++f++"\","++show l++")"
 instr (PUSHBT fn f l) = "PUSHBT(L\""++fn++"\",L\""++f++"\","++show l++")"
-instr (INLAM f) = "INLAM(\""++f++"\")"
+instr (INLAM f) = "INLAM(L\""++f++"\")"
 instr POPBT = "POPBT"
 instr (CHECKCACHE n) | n<5 = "CHECKCACHE("++show n++")"
 		     | otherwise = ""
@@ -367,13 +386,12 @@ mkfcall n lib ty args = popvals 0 (length args) ++
 
 {- CIM 12/7/05 changed to KVOID to avoid clashes with MinGW -}
     conv (Prim Void) = "KVOID("
-    conv (Prim Number) = "PUSH(MKINT"
+    conv (Prim Number) = "DECLAREQUICK(fr);STACKINT(fr,"
     conv (Prim RealNum) = "PUSH(MKREAL"
-    conv (Prim Boolean) = "PUSH(MKINT"
-    conv (Prim Character) = "PUSH(MKCHAR"
+    conv (Prim Boolean) = "DECLAREQUICK(fr);STACKINT(fr,"
+    conv (Prim Character) = "DECLAREQUICK(fr);STACKINT(fr,(kint)"
     conv (Prim StringType) = "PUSH(MKSTR"
-    conv (Prim File) = "PUSH(MKINT"
-    conv (Prim Pointer) = "PUSH(MKINT"
+    conv (Prim Pointer) = "DECLAREQUICK(fr);STACKRAW(fr,"
     conv (TyVar _) = "PUSH(" -- Enough rope to hang yourself with!
     conv (Array _) = "PUSH(MKARRAYVAL"
     conv t = "PUSH(" -- error $ "Can't deal with that type in foreign calls" ++ show t
@@ -387,8 +405,6 @@ mkfcall n lib ty args = popvals 0 (length args) ++
     stackconv' n (Prim Character) = show (tmpval n) ++ "->getInt()"
     stackconv' n (Prim StringType) 
 	= show (tmpval n) ++ "->getString()->getVal()"
-    stackconv' n (Prim File) 
-	= "(FILE*)("++show (tmpval n) ++ "->getRaw())"
     stackconv' n (Prim Pointer) = show (tmpval n) ++ "->getRaw()"
     stackconv' n (Array _) = show (tmpval n) ++ "->getArray()"
     stackconv' n (TyVar _) = show (tmpval n)
