@@ -14,6 +14,8 @@ import Options
 import Language
 import InfGadgets
 import VarUsage
+import PMComp
+import Module
 
 import Debug.Trace
 import List
@@ -23,6 +25,7 @@ import Control.Monad
 -- we know what all the variables are.
 
 mkpub (x,y) = (x,(y,[Public]))
+mkCtxtEntry opts (x,y) = (x,(y,opts))
 
 insertdecl :: Name -> -- Module
               EContext ->
@@ -32,84 +35,118 @@ insertdecl mod excs globs ctxt (RLambda str l tvs ns exp) =
       (RLambda str l tvs ns (insertdecl mod excs globs 
 			     (addNames ctxt (map mkpub ns)) exp))
 insertdecl mod excs globs ctxt t = 
-    declare (nubBy cmpnames (gu ((getNames ctxt)++(map fst globs)) t)) t
+    declare (nubBy cmpnames (gu False ((getNames ctxt)++(map fst globs)) t)) t
  -- Pass through list of vars declared
  where
    cmpnames (_,_,x) (_,_,y) = x==y
-   gu ds (RVar str l n) = checkdecl (str,l,n) ds []
-   gu ds (RLambda str l tvs ns exp) = 
-         gu (ds++(map fst ns)) exp
-   gu ds (RClosure str l ns exp) = 
-         gu (ds++(map fst ns)) exp
-   gu ds (RDeclare str l (n,loc) ty exp) =
-         gu (n:ds) exp
-   gu ds (RBind str l n ty val exp) =
-         {- gu ds val ++ -} gu (n:ds) exp
-   -- We want to check the rhs of an assignment, but not if it's a name 
-   gu ds (RAssign str l lval (RVar _ _ _)) 
-       = checkdecl (getlv lval) ds [] --(gu ds exp)
-   gu ds (RAssign str l lval exp) = checkdecl (getlv lval) ds (gu ds exp)
-   gu ds (RAssignOp str l op lval exp) = 
-       checkdecl (getlv lval) ds [] --(gu ds exp)
-   gu ds (RSeq str l lseq rseq) =
-       gu ds lseq ++ gu ds rseq
-   gu ds (RReturn _ _ (RCase str l v exps)) 
-       = {-gu ds v ++ -} (concat (map (gua ds) exps))
-   gu ds (RReturn str l a) = []
-   gu ds (RApply str l fn args) = []
+   gu mtch ds (RVar str l n) = checkdecl mtch (str,l,n) ds []
+   gu mtch ds (RQVar str l n) = checkdecl mtch (str,l,n) ds []
+   gu mtch ds (RLambda str l tvs ns exp) = 
+         gu mtch (ds++(map fst ns)) exp
+   gu mtch ds (RClosure str l ns exp) = 
+         gu mtch (ds++(map fst ns)) exp
+   gu mtch ds (RDeclare str l (n,loc) ty exp) =
+         gu mtch (n:ds) exp
+   gu mtch ds (RBind str l n ty val exp) =
+         {- gu mtch ds val ++ -} gu mtch (n:ds) exp
+   -- We want to check the rhs of an assignment, but not if it's a name.
+   -- We look because we might find a case statement or some kind of embedded
+   -- sequence with an assignment. We eliminate names at first because we don't
+   -- want a declaration inserted if this is the only place they are referred 
+   -- to!
+   gu mtch ds (RAssign str l lval (RVar _ _ _)) 
+       = checkdecl mtch (getlv lval) ds [] --(gu mtch ds exp)
+   gu mtch ds (RAssign str l lval (RQVar _ _ _)) 
+       = checkdecl mtch (getlv lval) ds [] --(gu mtch ds exp)
+   gu mtch ds (RAssign str l lval exp) = checkdecl mtch (getlv lval) ds (gu mtch ds exp)
+   gu mtch ds (RAssignOp str l op lval exp) = 
+       checkdecl mtch (getlv lval) ds [] --(gu mtch ds exp)
+   gu mtch ds (RSeq str l lseq rseq) =
+       gu mtch ds lseq ++ gu mtch ds rseq
+   gu mtch ds (RReturn _ _ (RCase str l v exps)) 
+       = {-gu mtch ds v ++ -} (concat (map (gua mtch ds) exps))
+   gu mtch ds (RReturn str l a) = []
+   gu mtch ds (RApply str l fn args) =
+       -- Names in applications must be declared or used already, except
+       -- in match patterns
+       if mtch then (concat (map (gu True ds) args))
+          else []
+   gu mtch ds (RPartial str l fn args) = []
        -- Names in applications must be declared or used already.
-       --checkdecl fn ds (concat (map (gu ds) args))
-   gu ds (RPartial str l fn args) = []
+       --checkdecl mtch fn ds (concat (map (gu mtch ds) args))
+   gu mtch ds (RForeign str l ty fn args) = []
        -- Names in applications must be declared or used already.
-       --checkdecl fn ds (concat (map (gu ds) args))
-   gu ds (RForeign str l ty fn args) = []
-       -- Names in applications must be declared or used already.
-       --checkdecl fn ds (concat (map (gu ds) args))
-       --concat (map (gu ds) args)
-   gu ds (RWhile str l c body) =
-       {-gu ds c ++ -} gu ds body
-   gu ds (RDoWhile str l body c) =
-       {-gu ds c ++ -} gu ds body
-   gu ds (RFor str l lval nm t body) =
-       checkdecl (getlv lval) ds (gu ds body)
---   gu ds (RFor str l lval (Just nm) t body) =
---       checkdecl (str,l,nm) ds $ checkdecl (getlv lval) ds (gu ds body)
-   gu ds (RTryCatch str l tr ca x fin) =
-       checkdecl (str,l,x) ds ((gu ds tr) ++ (gu ds ca) ++ (gu ds fin))
-   gu ds (RNewTryCatch str l tblock cs) =
-       gu ds tblock ++ (concat (map (gue ds) cs))
-   gu ds (RThrow str l a) = []
-   gu ds (RExcept str l a b) = []
-   gu ds (RPrint str l v) = [] --gu ds v
-   gu ds (RInfix str l op x y) = [] -- gu ds x ++ gu ds y
-   gu ds (RUnary str l op v) = [] -- gu ds v
-   gu ds (RCoerce str l t v) = [] --gu ds v
-   gu ds (RCase str l v exps) = {-gu ds v ++ -} (concat (map (gua ds) exps))
-   gu ds (RIf str l c t f) = {-gu ds c ++ -} gu ds t ++ gu ds f
-   gu ds (RIndex str l x y) = [] -- gu ds x ++ gu ds y
-   gu ds (RArrayInit str l xs) = [] --(concat (map (gu ds) xs))
-   gu ds x = []
+       --checkdecl mtch fn ds (concat (map (gu mtch ds) args))
+       --concat (map (gu mtch ds) args)
+   gu mtch ds (RWhile str l c body) =
+       {-gu mtch ds c ++ -} gu mtch ds body
+   gu mtch ds (RDoWhile str l body c) =
+       {-gu mtch ds c ++ -} gu mtch ds body
+   gu mtch ds (RFor str l lval nm (RVar _ _ _) body) =
+       checkdecl mtch (getlv lval) ds (gu mtch ds body)
+   gu mtch ds (RFor str l lval nm t body) =
+       checkdecl mtch (getlv lval) ds (gu mtch ds body ++ gu mtch ds t)
+--   gu mtch ds (RFor str l lval (Just nm) t body) =
+--       checkdecl mtch (str,l,nm) ds $ checkdecl mtch (getlv lval) ds (gu mtch ds body)
+   gu mtch ds (RTryCatch str l tr ca x fin) =
+       checkdecl mtch (str,l,x) ds ((gu mtch ds tr) ++ (gu mtch ds ca) ++ (gu mtch ds fin))
+   gu mtch ds (RNewTryCatch str l tblock cs) =
+       gu mtch ds tblock ++ (concat (map (gue mtch ds) cs))
+   gu mtch ds (RThrow str l a) = []
+   gu mtch ds (RExcept str l a b) = []
+   gu mtch ds (RPrint str l v) = [] --gu mtch ds v
+   gu mtch ds (RInfix str l op x y) = [] -- gu mtch ds x ++ gu mtch ds y
+   gu mtch ds (RUnary str l op v) = [] -- gu mtch ds v
+   gu mtch ds (RCoerce str l t v) = [] --gu mtch ds v
+   gu mtch ds (RCase str l v exps) = {-gu mtch ds v ++ -} 
+                                     (concat (map (gua mtch ds) exps))
+   gu mtch ds (RMatch str l v exps) = {-gu mtch ds v ++ -} 
+                                      (concat (map (guma ds) exps))
+   gu mtch ds (RIf str l c t f) = {-gu mtch ds c ++ -} 
+                                  gu mtch ds t ++ gu mtch ds f
+   gu mtch ds (RIndex str l x y) = [] -- gu mtch ds x ++ gu mtch ds y
+   gu mtch ds (RArrayInit str l xs) = 
+       -- Names in array initialisation must be declared or used already, 
+       -- except in match patterns
+       if mtch then (concat (map (gu True ds) xs))
+          else []
+   gu mtch ds x = []
 
    -- Look for variables used in case branches
-   gua ds (RAlt str l f as exp) = (cd str l as ds) ++ (gu ds exp)
-   gua ds (RConstAlt str l c exp) = gu ds exp
-   gua ds (RDefault str l exp) = gu ds exp
+   gua mtch ds (RAlt str l f as exp) = (cd mtch str l as ds) ++ (gu mtch ds exp)
+   gua mtch ds (RArrayAlt str l as exp) = (cd mtch str l as ds) ++ (gu mtch ds exp)
+   gua mtch ds (RConstAlt str l c exp) = gu mtch ds exp
+   gua mtch ds (RDefault str l exp) = gu mtch ds exp
+
+   guma ds (MAlt str l ps exp)
+       = (concat (map (gu True ds) ps)) ++ (gu False ds exp)
 
    -- Look for variables used in catch clauses
-   gue ds (RCatch str l (Left (ex,as)) exp) = (cd str l as ds) ++ (gu ds exp)
+   gue mtch ds (RCatch str l (Left (ex,as)) exp) = (cd mtch str l as ds) ++ (gu mtch ds exp)
    -- only insert the declaration if 'v' isn't a global name
-   gue ds (RCatch str l (Right v) exp) 
+   gue mtch ds (RCatch str l (Right v) exp) 
        = case lookupname mod v excs of
-            [] -> checkdecl (str,l,v) ds (gu ds exp)
-            _ -> gue ds (RCatch str l (Left (v,[])) exp)
+            [] -> checkdecl mtch (str,l,v) ds (gu mtch ds exp)
+            _ -> gue mtch ds (RCatch str l (Left (v,[])) exp)
    -- Check for whether each in a list of vars has been declared
    -- Return those which need declarations inserted
-   cd str l [] ds = []
-   cd str l (x:xs) ds = checkdecl (str,l,x) ds (cd str l xs ds)
+   cd mtch str l [] ds = []
+   cd mtch str l (x:xs) ds = checkdecl mtch (str,l,x) ds (cd mtch str l xs ds)
 
-   checkdecl (f,l,var) ds rest = case var `elem` ds of
-				   True -> rest
-				   False -> (f,l,var):rest
+   -- Add a declaration if we come across a variable that hasn't been declared.
+   -- If we're in a match pattern, also check whether the name is in global
+   -- scope.
+   checkdecl mtch (f,l,var) ds rest 
+       = case (var `elem` ds, mtch, lookupname mod var ctxt) of
+             -- in local scope, do nothing
+	    (True, _, _) -> rest
+             -- in global scope, and we're in a match
+            (_, True, xs) -> if (all nonConstructor xs) then (f,l,var):rest
+                                else rest
+             -- not in local scope, so add it (don't care about global scope)
+	    (False, _, _) -> (f,l,var):rest
+
+   nonConstructor (n,(ty,opts)) = not (Constructor `elem` opts)
 
    getlv (RAName str l x) = (str,l,x)
    getlv (RAIndex str l lv t) = getlv lv
@@ -157,7 +194,7 @@ infertype mod ctxt globs excs tags tys flds t ftype vtype copts = do
    -- reverse the environment to get most recent binding first.
       case (lookup n (reverse env)) of
         Nothing -> findfun n
-        (Just t) -> do let (nt, next') = fudgevars t next -- NO! Only generalise type variables for global functions!
+        (Just t) -> do let (nt, next') = (t,next) -- fudgevars t next -- NO! Only generalise type variables for global functions!
                        let (realt, realn) = isapply nt (Global n "" (argSpace t))
 		       neweqn <- addEq file line (realt,exp,"identifier '" ++ showuser n++"'") eqns
 		       return (realn, neweqn, next')
@@ -177,14 +214,14 @@ infertype mod ctxt globs excs tags tys flds t ftype vtype copts = do
 		     (Just (t,i)) -> do
 		         neweqn <- addEq file line (t,exp,"identifier '" ++ showuser n ++ "'") eqns
 			 return (GVar i, neweqn, next)
-	   isapply t@(Fn [] [] ty) f = (ty,Apply f [])
+	   isapply t@(Fn [] [] ty) f = (ty,Annotation (Line file line) (Apply f []))
 	   isapply t f = (t,f)
    tiVar env next (RQVar file line n) eqns exp disambig rexp =
    -- reverse the environment to get most recent binding first.
    -- trace ("*** trying " ++ (show n) ++ " : " ++ (show exp)) $
       case (lookup n (reverse env)) of
         Nothing -> findfun n
-        (Just t) -> do let (nt, next') = fudgevars t next
+        (Just t) -> do let (nt, next') = (t,next) -- fudgevars t next
 		       neweqn <- addEq file line (nt,exp,"identifier '" ++ showuser n ++ "'") eqns
 		       return (Global n "" (argSpace t), neweqn, next')
       where findfun n = case (ctxtlookup mod n ctxt disambig copts) of
@@ -287,10 +324,10 @@ infertype mod ctxt globs excs tags tys flds t ftype vtype copts = do
          (ti env (next+1) f eqns newtv rexp)
          (do -- need args to help us disambiguate f
              let (pre_atypesin, next') = mkatypes args (next+1)
-             (_, pre_aeqns, next) <- checkargs args pre_atypesin eqns next'
+             (_, pre_aeqns, next'') <- checkargs args pre_atypesin eqns next'
              phi <- mkSubst pre_aeqns
              let calledFnType = Fn [] (map (subst phi) pre_atypesin) newtv
-             tiVar env next f eqns newtv (Just calledFnType) rexp)
+             tiVar env next'' f eqns newtv (Just calledFnType) rexp)
       phi <- mkSubst feqns
       let ftype = subst phi newtv
       let (Fn defs argtypes _) = ftype
@@ -304,8 +341,9 @@ infertype mod ctxt globs excs tags tys flds t ftype vtype copts = do
       (achecks, aeqns, next'') <- checkargs args' atypes neweqns next''
 --      (finf,feqns,next'') <- ti env next' (RQVar file line f) aeqns reqtype
       let app = case fcheck of
-                   False -> Apply finf achecks
-                   True -> Annotation (DynCheck exp) (Apply finf achecks)
+                   False -> (Apply finf achecks)
+                   True -> Annotation (DynCheck exp) 
+                               (Apply finf achecks)
 --      neweqns <- addEq file line (reqtype, ftype) aeqns
       return (Annotation (Line file line) app, aeqns, next'')
      where mkatypes :: [Raw] -> Int -> ([Type],Int)
@@ -495,6 +533,21 @@ infertype mod ctxt globs excs tags tys flds t ftype vtype copts = do
       (alts,aeqns,next'') <- altinf env next' alts veqns vtv exp rexp
       return (Annotation (Line file line) (Case vinf (sort alts)), 
                          aeqns, next'')
+   ti env next (RMatch file line v alts) eqns exp rexp = do
+      -- Check the alternatives before we do anything, because then the
+      -- error message will be based on the actual code, not the transformed
+      -- code. It's a bit wasteful, because we have to do it again anyway,
+      -- but at least it gives meaningful feedback, and helps verify that
+      -- the match compiler is correct,
+      let vtv = TyVar (MN ("TYPE",next))
+      (eqns', next') <- verifyAlts env (next+1) alts eqns vtv exp rexp
+      case (mkSubst eqns') of
+          Failure e f l -> fail e
+          _ -> return ()
+      mapM_ (checkLinear mod ctxt) alts
+      let (simpleCase, next') = mkCase file line ctxt mod next v alts
+      {- trace (show simpleCase) $ -} 
+      ti env next simpleCase eqns exp rexp
    ti env next (RIf file line cond ift iff) eqns exp rexp = do
       (cinf, ceqns, next') <- ti env next cond eqns (Prim Boolean) rexp
       (tinf, teqns, next'') <- ti env next' ift ceqns exp rexp
@@ -529,14 +582,32 @@ infertype mod ctxt globs excs tags tys flds t ftype vtype copts = do
    ti env next (RNoop file line) eqns exp rexp = do
        neweqns <- addEq file line (Prim Void, exp, "no-op") eqns
        return (Noop, neweqns, next)
+   ti env next (RUnderscore file line) eqns exp rexp = 
+       -- placeholder - this never gets compiled.
+       return (Noop, eqns, next)
    ti env next (RVMPtr file line) eqns exp rexp = do
        neweqns <- addEq file line (Prim Pointer, exp, "VM pointer") eqns
        return (VMPtr, neweqns, next)
+   ti env next (RLength file line str) eqns exp rexp = do
+       eqns' <- addEq file line (Prim Number, exp, "string length") eqns 
+       (strinf, streqns, next') <- ti env (next+1) str eqns' (Prim StringType) rexp
+       return (Length strinf, streqns, next')
    ti env next (RMetavar f l i) eqns exp rexp = 
        return (Metavar f l i, eqns, next)
        
 --   ti env next x eqns exp = error $ show x
 
+   verifyAlts env next [] eqns vt exp rexp = return (eqns, next)
+   -- the parser will only ever give us one pattern, this only becomes a list
+   -- during intermediate stages (for the moment). Don't check the return
+   -- value, that'll get done later.
+   -- All we're doing here is checking that the patterns are all consistent
+   -- with each other.
+   verifyAlts env next ((MAlt _ _ [p] res):ms) eqns vt exp rexp
+      = do (pinf, peqns, next') <- ti env next p eqns vt rexp
+           (eqns', next'') <- verifyAlts env next' ms peqns vt exp rexp
+           return (eqns', next'')
+
    altinf env next [] eqns vt exp rexp = return ([],eqns,next)
    altinf env next ((RDefault file line r):xs) eqns vt exp rexp
     = do (xsinf, xseqns, next') <- altinf env next xs eqns vt exp rexp
@@ -556,6 +627,13 @@ infertype mod ctxt globs excs tags tys flds t ftype vtype copts = do
 	   checkCaseable StringType _ _ = return ()
 	   checkCaseable x f l = fail $ f ++":"++show l++":" ++
 		   "Can't do case analysis on " ++ show (Prim x)
+   altinf env next ((RArrayAlt file line args r):xs) eqns vt exp rexp 
+    = do let arr = RArrayInit file line (map (RVar file line) args)
+         (arinf, areqns, next') <- ti env next arr eqns vt rexp
+         (xsinf, xseqns, next'') <- altinf env next' xs areqns vt exp rexp
+         (rinf, reqns, next''') <- ti env next'' r xseqns exp rexp
+         return ((ArrayAlt (map (\x -> Global x file line) args) rinf):xsinf,
+                  reqns, next''')
    altinf env next ((RAlt file line con args r):xs) eqns vt exp rexp 
     = do
        conname <- case ctxtlookup mod con ctxt Nothing copts of
@@ -586,10 +664,10 @@ infertype mod ctxt globs excs tags tys flds t ftype vtype copts = do
           Got (fulln, _) ->
               exinf env next ((RCatch f l (Left (x, [])) handler):xs) eqns exp rexp
           _ -> do (rest, eqns', next') <- exinf env next xs eqns exp rexp
+                  -- 'rest' will never get caught, so don't compile it.
                   (hinf, heqns, next'') <- ti env next' handler eqns' exp rexp
                   (xinf, xeqns, next''') <- ti env next'' (RVar f l x) heqns (Prim Exception) rexp
-                  return ((Catch (Right xinf) hinf):rest, xeqns, next''')
-
+                  return ((Catch (Right xinf) hinf):[], xeqns, next''')
    checkExArgs f l [] [] env eqns next rexp = return ([], eqns, next)
    checkExArgs f l (ty:tys) (x:args) env eqns next rexp = do
       (rest, eqns', next') <- checkExArgs f l tys args env eqns next rexp
@@ -710,7 +788,8 @@ appsyn tys t = {- trace ((show t) ++ " to " ++ show (syn t)) $ -} syn t where
     syn (Prim p) = (Prim p)
     syn (Fn ns ts t) = Fn ns (map syn ts) (syn t)
     syn (Array t) = Array (syn t)
-    syn (User n ts) = User (resolve tys n) (map syn ts)
+    syn (User n) = User (resolve tys n)
+    syn (TyApp n ts) = TyApp (syn n) (map syn ts)
 {-       where resolve [] n = n
 	     resolve ((cn@(NS _ bn),ti):xs) n | bn == n = cn
 	     resolve ((x,ti):xs) n | x == n = x
@@ -754,8 +833,9 @@ mkS' errs ((t1,t2,f,l):xs) = do
 -- Step 4: Substitute the type equations into the term with the substitution
 -- we got from the last step.
 
-substTerm :: Monad m => Subst -> Expr Name -> m (Expr Name)
-substTerm phi t = st t
+substTerm :: Monad m => Name -> -- Module name
+             Context -> Subst -> Expr Name -> m (Expr Name)
+substTerm mod ctxt phi t = st t
    where
      st (Lambda iv args exp) = do exp' <- st exp
 				  return $ Lambda iv (lamsubst args) exp'
@@ -865,7 +945,7 @@ substTerm phi t = st t
      st (InferInfix op e1 e2 (t1,t2,t3) f l) = do
 	 e1' <- st e1
 	 e2' <- st e2
-	 checkInfix op e1' e2' (subst phi t1) (subst phi t2) (subst phi t3) f l
+	 checkInfix mod ctxt op e1' e2' (subst phi t1) (subst phi t2) (subst phi t3) f l
 -- 	InferInfix op (st e1) (st e2) (subst phi t1,subst phi t2,subst phi t3)
 -- 		   f l
      st (Append e1 e2) = do
@@ -896,6 +976,11 @@ substTerm phi t = st t
 		       exps' <- mapM st exps
 		       rest <- stalt xs
 		       return $ (Alt n t exps' ex'):rest
+		stalt ((ArrayAlt exps ex):xs) = 
+		    do ex' <- st ex
+		       exps' <- mapM st exps
+		       rest <- stalt xs
+		       return $ (ArrayAlt exps' ex'):rest
      st (ArrayInit e) = do e' <- mapM st e
 			   return $ ArrayInit e'
      st (If e1 e2 e3) = do e1' <- st e1
@@ -914,55 +999,66 @@ substTerm phi t = st t
 			      return $ Annotation a e'
      st x = return x
 
-checkInfix :: Monad m => Op -> Expr Name -> Expr Name -> 
+checkInfix :: Monad m => Name -> Context -> Op -> Expr Name -> Expr Name -> 
 	                 Type -> Type -> Type -> 
 			 String -> Int -> m (Expr Name)
 -- If the types are different, try to insert a coercion
-checkInfix op l r (Prim x) (Prim y) ret file line
+checkInfix mod ctxt op l r (Prim x) (Prim y) ret file line
     | x `tlt` y = do col <- implicitCoerce x y l file line
-		     checkInfix op col r (Prim y) (Prim y) ret file line
+		     checkInfix mod ctxt op col r (Prim y) (Prim y) ret file line
     | y `tlt` x = do cor <- implicitCoerce y x r file line
-		     checkInfix op l cor (Prim x) (Prim x) ret file line
-checkInfix op l r (Prim StringType) (Prim StringType) (Prim Boolean) file line
+		     checkInfix mod ctxt op l cor (Prim x) (Prim x) ret file line
+checkInfix mod ctxt op l r (Prim StringType) (Prim StringType) (Prim Boolean) file line
   | (op==Equal || op == NEqual) =
     return $ CmpStr op l r
-checkInfix Plus l r (Prim StringType) (Prim StringType) (Prim StringType) file line
+checkInfix mod ctxt Plus l r (Prim StringType) (Prim StringType) (Prim StringType) file line
  = return $ Append l r
-checkInfix op l r (Prim Number) (Prim Number) (Prim Number) file line
+checkInfix mod ctxt op l r (Prim Number) (Prim Number) (Prim Number) file line
   | (op==Plus || op==Minus || op==Times || op==Divide || op==Modulo || 
      op==Power || op==OpAnd || op==OpOr || op==OpXOR || 
      op==OpShLeft || op==OpShRight)
       = return $ Infix op l r
-checkInfix op l r (Prim RealNum) (Prim RealNum) (Prim RealNum) file line
+checkInfix mod ctxt op l r (Prim RealNum) (Prim RealNum) (Prim RealNum) file line
   | (op==Plus || op==Minus || op==Times || op==Divide || 
      op==Power || op==OpAnd || op==OpOr)
       = return $ RealInfix op l r
-checkInfix op l r (Prim Number) (Prim Number) (Prim Boolean) file line
+checkInfix mod ctxt op l r (Prim Number) (Prim Number) (Prim Boolean) file line
   | (op==Equal || op==NEqual || op==OpLT || op==OpGT ||
      op==OpLE || op==OpGE)
       = return $ Infix op l r
-checkInfix op l r (Prim RealNum) (Prim RealNum) (Prim Boolean) file line
+checkInfix mod ctxt op l r (Prim RealNum) (Prim RealNum) (Prim Boolean) file line
   | (op==Equal || op==NEqual || op==OpLT || op==OpGT ||
      op==OpLE || op==OpGE)
       = return $ RealInfix op l r
-checkInfix op l r (Prim Character) (Prim Character) (Prim Boolean) file line
+checkInfix mod ctxt op l r (Prim Character) (Prim Character) (Prim Boolean) file line
   | (op==Equal || op==NEqual || op==OpLT || op==OpGT ||
      op==OpLE || op==OpGE)
       = return $ Infix op l r
-checkInfix op l r (Prim Boolean) (Prim Boolean) (Prim Boolean) file line
+checkInfix mod ctxt op l r (Prim Boolean) (Prim Boolean) (Prim Boolean) file line
   | (op==Equal || op==NEqual || op==OpAndBool || op==OpOrBool)
       = return $ Infix op l r
-checkInfix op l r (Prim Exception) (Prim Exception) (Prim Boolean) file line
+checkInfix mod ctxt op l r (Prim Exception) (Prim Exception) (Prim Boolean) file line
   | (op==Equal || op==NEqual)
       = return $ CmpExcept op l r
 
 -- Defaults for equality, pass through to built in equal function
-checkInfix Equal l r t1 t2 (Prim Boolean) file line
+checkInfix mod ctxt Equal l r t1 t2 (Prim Boolean) file line
       | t1==t2 = return $ Apply (Global eqfun eqmangle 2) [l,r]
-checkInfix NEqual l r t1 t2 (Prim Boolean) file line
+checkInfix mod ctxt NEqual l r t1 t2 (Prim Boolean) file line
       | t1==t2 = return $ Unary Not (Apply (Global eqfun eqmangle 2) [l,r])
+{-
+Operator overloading: It works but I'm still not switching it on until
+I've either worked out proper general overloading or decided we shouldn't
+worry about it.
+checkInfix mod ctxt op l r tl tr ret file line
+   | Just (nm,ty@(Fn _ _ rty)) <- 
+       ctxtlookup mod (OP op) ctxt (Just (Fn [] [tl,tr] ret)) []
+      = if (rty == ret) then 
+           return $ Apply (Global nm (mangling ty) (argSpace ty)) [l,r]
+           else fail $ file ++ ":" ++ show line ++ ":Can't apply operator '" ++ show op ++ "' to " ++ show tl ++ " and " ++ show tr ++ " to give " ++ show ret
+-}
 
-checkInfix op l r tl tr ret file line = fail $ file ++ ":" ++ show line ++ ":Can't apply operator '" ++ show op ++ "' to " ++ show tl ++ " and " ++ show tr ++ " to give " ++ show ret
+checkInfix mod ctxt op l r tl tr ret file line = fail $ file ++ ":" ++ show line ++ ":Can't apply operator '" ++ show op ++ "' to " ++ show tl ++ " and " ++ show tr ++ " to give " ++ show ret
 
 checkUnary :: Monad m => UnOp -> Expr Name -> 
 	                 Type -> Type -> 
@@ -1076,7 +1172,7 @@ inferAll mod ctxt errs globs excs tags syns flds dump ((FB (file,line,x,xt,fopts
 	        xeq' <- return $! 
                           (if dump then (trace (showeqns x xft' xeq) xeq) else xeq)
 	        phi' <- {- trace (show xrv) $ -} mkSubst xeq'
-	        xfn <- substTerm phi' xrv
+	        xfn <- substTerm mod ctxt phi' xrv
 	        let xinft = subst phi' xft'
 	        checkEq file line xinft xft'
 	        checkReturn file line (showuser x) xft' xfn
@@ -1125,7 +1221,7 @@ inferAll mod ctxt errs globs excs tags syns flds dump ((FB (file,line,x,xt,fopts
              return (rest, err:errs, ctxt', globs', excs', tags', syns')
          Success (xrv,xeq) -> do
              phi' <- {- trace (show xrv) $ -} mkSubst xeq
-             xfn <- substTerm phi' xrv
+             xfn <- substTerm mod ctxt phi' xrv
              let xinft = subst phi' xft
              checkEq file line xinft xft
              checkReturn file line (showuser x) xft xfn
@@ -1144,7 +1240,7 @@ inferAll mod ctxt errs globs excs tags tydefs flds dump ((TSyn (file,line,n,ps,t
      (rest, errs, ctxt', globs', excs', tags',syns') <- 
 	 inferAll mod ctxt errs globs excs tags newdefs flds dump xs copts
      -- normalise to check for cycles
-     foo <- normalise file line mod syns' (User n ps)
+     foo <- normalise file line mod syns' (TyApp (User n) ps)
      return $ ((TySyn (file,line,n,(map getname ps),ty,exp):rest), errs, 
                ctxt', globs', excs', tags', syns')
   where getname (TyVar x) = x
@@ -1165,7 +1261,8 @@ inferAll mod ctxt errs globs excs tags tydefs flds dump ((DDecl f l dopts n tys
 	then fail $ f ++ ":" ++ show l ++ ": Data type " ++ 
 	            showuser n ++ " already defined"
 	else return ()
-     let ctxt' = addNames ctxt (map mkpub (gettys newdefs fbs))
+     let ctxt' = addNames ctxt (map (mkCtxtEntry [Public,Constructor]) 
+                                        (gettys newdefs fbs))
      let tags' = (gettags fbs (length cons))++tags
      (rest, errs, ctxt'', globs'', excs'', tags'', syns') 
          <- inferAll mod ctxt' errs globs excs tags' newdefs 
@@ -1265,7 +1362,7 @@ getFuns ctxt f l mod ((Con n ty an codegen):xs) syns i
 	  else return () -- Do nothing if it's from somewhere else
        normty <- normalise f l mod syns (appsyn syns ty)
      --  trace (showuser n ++ ":" ++ show normty) $ return ()
-       return (((FunBind (f,l,n,normty,[Inline,Pure,Public],DataCon i (getarglen ty) codegen) "" normty):rest),
+       return (((FunBind (f,l,n,normty,[Inline,Pure,Public,Constructor],DataCon i (getarglen ty) codegen) "" normty):rest),
 	       (getFields (appsyn syns ty) an 0 i) ++ flds, codegen)
   where getarglen (Fn _ tys _) = length tys
 	getarglen _ = 0
@@ -1315,7 +1412,8 @@ checkTyVars f l ts t = ct t where
    ct (Fn ds ts t) = do mapM_ ct ts
 			ct t
    ct (Array t) = ct t
-   ct (User n ts) = mapM_ ct ts
+   ct (TyApp t ts) = do ct t
+                        mapM_ ct ts
    ct t@(TyVar _) | t `elem` ts = return ()
 		  | otherwise = fail $ f ++ ":" ++ show l ++ ":" ++
 				          show t ++ 
@@ -1336,18 +1434,15 @@ checkConDecl f l syns tys (Con n ty _ _) =
        allDeclared f l n syns tys ty
 
 allDeclared f l n dtys tys ty = {- trace (show ty) $ -} ad ty
-  where ad (Fn _ ts t) = adM (t:ts)
+  where ad (Fn _ ts t) = mapM_ ad (t:ts)
 	ad (Array t) = ad t
-	ad (User n ts) = adM ts
+	ad (TyApp t ts) = do ad t
+                             mapM_ ad ts
 	ad (TyVar x) | (TyVar x) `elem` tys = return ()
 		     | otherwise = fail $ f ++ ":" ++ show l ++ ":" ++
 				   "undeclared type variable " ++ showuser x
 	ad _ = return ()
 
-        adM [] = return ()
-	adM (x:xs) = do ad x
-			adM xs
-
 checkUserData :: Monad m => String -> Int -> Name -> -- Module
 		 Types -- Synonyms/Datatypes
 	     -> [ConDecl] -- Constructor declarations
@@ -1361,19 +1456,17 @@ checkUserDecl f l mod syns (Con n ty _ _)
        userDeclared f l n syns normty
 
 userDeclared f l n dtys ty = ad ty
-  where ad (Fn _ ts t) = adM (t:ts)
+  where ad (Fn _ ts t) = mapM_ ad (t:ts)
 	ad (Array t) = ad t
-	ad (User n ts) = case (typelookup (UN "foo") n dtys) of
-			    (Got _) -> adM ts
-			    _ -> fail $ f ++ ":" ++ show l ++ ":" ++
-                                        "unknown type "++ showuser n
+        ad (User n) = case (typelookup (UN "foo") n dtys) of
+                         (Got _) -> return ()
+			 _ -> fail $ f ++ ":" ++ show l ++ ":" ++
+                                     "unknown type "++ showuser n
+	ad (TyApp t ts) = do ad t
+                             mapM_ ad ts
 	ad (TyVar x) = return ()
 	ad _ = return ()
 
-        adM [] = return ()
-	adM (x:xs) = do ad x
-			adM xs
-
 checkGlob :: MonadPlus m => String -> Int ->
                   Name -> Context -> GContext -> EContext -> Tags -> Types ->
                   Fields -> Maybe Raw -> Type -> Options -> m (Maybe (Expr Name))
@@ -1408,8 +1501,10 @@ getVarTypes t1 t2 ctxt = do tvm <- cg t1 t2 []
      cg (Fn ns ts t) (Fn ns' ts' t') tvm = do
           tvm' <- cg t t' tvm
 	  cgl ts ts' tvm'
-     cg (User n ts) (User n' ts') tvm | n == n' = cgl ts ts' tvm
-				      | otherwise = fail "Type error"
+     cg (User n) (User n') tvm | n == n' = return tvm
+                               | otherwise = fail "Type error"
+     cg (TyApp t ts) (TyApp t' ts') tvm = do tvm' <- cg t t' tvm
+                                             cgl ts ts' tvm'
      cg (Prim x) (Prim y) tvm | x == y = return tvm
 			      | otherwise = fail "Type error"
      cg _ _ tvm = fail "Type error"
@@ -1417,6 +1512,7 @@ getVarTypes t1 t2 ctxt = do tvm <- cg t1 t2 []
      cgl [] [] tvm = return tvm
      cgl (x:xs) (y:ys) tvm = do tvm' <- cg x y tvm
 				cgl xs ys tvm'
+     cgl _ _ tvm = fail "Type error"
 
 
 checkRpt :: Monad m => String -> Int -> Name -> Type -> Context -> Bool -> m ()
