@@ -86,7 +86,7 @@ lift nm ty exp = lift' [] [] exp where
        let (defs''',lval') = liftlval defs'' locs lval in
 	   (defs''',For i nm j lval' e1' e2')
      -- fake variables to account for For introducing names into environement
-     where fakevars = [(MN ("_",0), Prim Number), (MN ("_",0), UnknownType)]
+     where fakevars = [(MN ("_",i), Prim Number), (MN ("_",j), UnknownType)]
    lift' defs locs (TryCatch e1 e2 e3 e4) =
        let (defs',e1') = lift' defs locs e1 in
        let (defs'',e2') = lift' defs' locs e2 in
@@ -189,6 +189,8 @@ lift nm ty exp = lift' [] [] exp where
       where getty ((_,ty,_):xs) = ty
             renamelocs xs = rl 0 xs
 	    rl _ [] = []
+            -- skip machine generated names, they won't be used
+--            rl n (((MN _),_):xs) = rl n xs
 	    rl n ((nm,ty):xs) = ((rln n nm,ty):(rl (n+1) xs))
 	    rln n (UN s) = MN (s,n)
 	    rln n (NS mod nm) = (NS mod (rln n nm))
@@ -212,6 +214,11 @@ lift nm ty exp = lift' [] [] exp where
 	   (defs'',e') = lift' defs' locs e
 	   (defs''',xs') = liftalts defs'' locs xs in
 	   (defs''',(Alt t i es' e'):xs')
+   liftalts defs locs ((ArrayAlt es e):xs) =
+       let (defs',es') = lifts' defs locs es
+	   (defs'',e') = lift' defs' locs e
+	   (defs''',xs') = liftalts defs'' locs xs in
+	   (defs''',(ArrayAlt es' e'):xs')
    liftalts defs locs ((ConstAlt pt c e):xs) =
        let (defs',e') = lift' defs locs e
 	   (defs'',xs') = liftalts defs' locs xs in
