@@ -122,9 +122,11 @@ ctxtlookup mod n ctx ty copts
         matchArgs (TyVar _) _ = True
         matchArgs _ (TyVar _) = True
         matchArgs (Array x) (Array y) = matchArgs x y
-        matchArgs (User t ts) (User u us) = t == u &&
+        matchArgs (TyApp t ts) (TyApp u us) = matchArgs t u &&
                                               (and $ zipWith matchArgs ts us)
-        matchArgs (Fn _ _ _) (Fn _ _ _) = True
+        matchArgs (Fn _ args1 _) (Fn _ args2 _) 
+            = (length args1 == length args2) && 
+              and (zipWith matchArgs args1 args2)
         matchArgs _ _ = False
 
 	-- A private name in the current module is accessible
@@ -165,14 +167,14 @@ mangling :: Type -> String
 mangling t = "_" ++ mangling' t
 mangling' (Fn _ args _) = "F" ++ concat (map mangling' args)
 mangling' (Array arg) = "a" ++ mangling' arg
-mangling' (User n args) = show n ++ concat (map mangling' args)
+mangling' (User n) = show n
+mangling' (TyApp n args) = mangling' n ++ concat (map mangling' args)
 mangling' (TyVar _) = "P"
 mangling' (Prim Number) = "i"
 mangling' (Prim Character) = "c"
 mangling' (Prim Boolean) = "b"
 mangling' (Prim RealNum) = "f"
 mangling' (Prim StringType) = "s"
-mangling' (Prim File) = "F"
 mangling' (Prim Pointer) = "p"
 mangling' (Prim Exception) = "e"
 mangling' (Prim Void) = "v"
@@ -182,7 +184,7 @@ mangling' _ = ""
 
 getTyVars :: Type -> [Name]
 getTyVars (TyVar n) = [n]
-getTyVars (User _ tys) = concat (map getTyVars tys)
+getTyVars (TyApp f tys) = concat (map getTyVars (f:tys))
 getTyVars (Array t) = getTyVars t
 getTyVars (Fn _ tys t) = concat (map getTyVars tys) ++ getTyVars t
 getTyVars _ = []
@@ -212,7 +214,7 @@ getVars :: Type -> [Name]
 getVars = nub.gv
     where gv (Fn ns ts t) = concat (map gv (t:ts))
 	  gv (Array t) = gv t
-	  gv (User n ts) = concat (map gv ts)
+	  gv (TyApp n ts) = concat (map gv (n:ts))
 	  gv (TyVar n) = [n]
 	  gv _ = []
 
@@ -237,7 +239,9 @@ fudgevars t next = let (vsmap,next') = newnames next (getUserVars t) in
 	  alpha vsmap (Fn ns tys t) = Fn ns (map (alpha vsmap) tys)
 				            (alpha vsmap t)
 	  alpha vsmap (Array t) = Array (alpha vsmap t)
-	  alpha vsmap (User n tys) = User n (map (alpha vsmap) tys)
+          alpha vsmap (User n) = User n
+	  alpha vsmap (TyApp n tys) = tyapp (alpha vsmap n) 
+                                            (map (alpha vsmap) tys)
 --	  alpha vsmap (Syn n) = Syn n
 	  alpha vsmap (TyVar x) = case lookup x vsmap of
 				     Nothing -> TyVar x
@@ -248,7 +252,7 @@ getUserVars :: Type -> [Name]
 getUserVars = nub.gv
     where gv (Fn ns ts t) = concat (map gv (t:ts))
 	  gv (Array t) = gv t
-	  gv (User n ts) = concat (map gv ts)
+	  gv (TyApp n ts) = concat (map gv (n:ts))
 	  gv (TyVar (UN n)) = [UN n]
 	  gv _ = []
 
@@ -298,41 +302,55 @@ exceptlookup mod e ei = returnName (nubnames (lookupname mod e ei))
 
 -- Type normalisation; expand synonyms.
 
+tyapp u [] = u
+tyapp u ts = TyApp u ts
+
 normalise :: Monad m => String -> Int -> Name -> Types -> Type -> m Type
-normalise f l mod ti t = tn [] t
+normalise f l mod ti t = tn True [] t
  where
-   tn u (Fn ds ts t) 
-       = do ts' <- mapM (tn u) ts
-	    t' <- tn u t
+   tn top u (Fn ds ts t) 
+       = do ts' <- mapM (tn top u) ts
+	    t' <- tn top u t
 	    return $ Fn ds ts' t'
-   tn u (Array t) = do t' <- tn u t
-		       return $ Array t'
-   tn u t@(User n ts) = 
+   tn top u (Array t) = do t' <- tn top u t
+		           return $ Array t'
+   tn top u t@(User n) = 
        case typelookup mod n ti of
           un@(UnknownName _) -> 
-              do ts' <- mapM (tn u) ts
-                 return $ User (fixup mod n) ts'
+               return $ User (fixup mod n)
           am@(Ambiguous n xs) -> fail $ f ++ ":" ++ show l ++ ":" ++ show am
-	  (Got (fqn, x)) -> applyTI u fqn ts x
-   tn u rest = return rest
+	  (Got (fqn, x)) -> applyTI top u fqn [] x
+   tn top u (TyApp (User n) []) = tn top u (User n)
+   tn top u t@(TyApp (User n) ts) = 
+       case typelookup mod n ti of
+          un@(UnknownName _) -> 
+               do ts' <- mapM (tn False u) ts
+                  return $ tyapp (User (fixup mod n)) ts'
+          am@(Ambiguous n xs) -> fail $ f ++ ":" ++ show l ++ ":" ++ show am
+	  (Got (fqn, x)) -> applyTI top u fqn ts x
+   tn top u (TyApp n ts) =
+       do ts' <- mapM (tn top u) ts
+          n' <- tn top u n 
+          return $ tyapp n' ts'
+   tn top u rest = return rest
 
    fixup m fqn@(NS _ _) = fqn
    fixup m n = (NS m n)
 --   fixup m n = n
 
-   applyTI u n ts (UserData as)
-	| length ts < length as 
+   applyTI top u n ts (UserData as)
+	| top && length ts < length as 
 	    = fail $ f ++ ":" ++ show l ++ ":" ++
 	       "Type " ++ showuser n ++ " has too few parameters"
-	| length ts > length as 
+	| top && length ts > length as 
 	    = fail $ f ++ ":" ++ show l ++ ":" ++
 	       "Type " ++ showuser n ++ " has too many parameters"
-	| otherwise = do ts' <- mapM (tn u) ts
-			 return $ User n ts'
+	| otherwise = do ts' <- mapM (tn False u) ts
+			 return $ tyapp (User n) ts'
    -- Replace type with 't', replacing instances of as inside t with
    -- corresponding instances of ts.
    -- That probably makes no sense.
-   applyTI u n ts (Syn as t)
+   applyTI top u n ts (Syn as t)
 	| length ts < length as
 	    = fail $ f ++ ":" ++ show l ++ ":" ++
 	       "Type synonym " ++ showuser n ++ " has too few parameters"
@@ -344,11 +362,11 @@ normalise f l mod ti t = tn [] t
 			     "Cycle in type synonyms " ++ 
 			     showsyns u
 		       else do st <- substty (zip as ts) t
-			       tn (n:u) st
+			       tn top (n:u) st
      where showsyns [n] = showuser n
 	   showsyns (n:ns) = showuser n ++ ", " ++ showsyns ns
 
-   applyTI u n ts Private = fail $ f ++ ":" ++ show l ++ 
+   applyTI top u n ts Private = fail $ f ++ ":" ++ show l ++ 
 			     "Can't use private type " ++ showuser n
 
    substty tmap (TyVar n) = case lookup n tmap of
@@ -359,8 +377,9 @@ normalise f l mod ti t = tn [] t
    substty tmap (Fn ds as r) = do as' <- mapM (substty tmap) as
 				  r' <- substty tmap r
 				  return $ Fn ds as' r'
-   substty tmap (User n ts) = do ts' <- mapM (substty tmap) ts
-				 return $ User n ts'
+   substty tmap (TyApp n ts) = do ts' <- mapM (substty tmap) ts
+                                  n' <- substty tmap n
+				  return $ tyapp n' ts'
    substty _ rest = return rest
 
 -- Fold constants in a raw term
@@ -427,6 +446,27 @@ foldunreal :: UnOp -> Double -> Maybe Const
 foldunreal Neg x = Just $ Re (-x)
 foldunreal _ _ = Nothing
 
+-- compile time coercions (only do the ones which the type checker accepts)
+
+mkInt :: Const -> Int
+mkInt (Num x) = x
+mkInt (Ch c) = fromEnum c
+mkInt (Bo False) = 0
+mkInt (Bo True) = 1
+mkInt (Re x) = fromEnum x
+mkInt (Str str) = read str
+mkInt e = error $ "This can't happen: mkInt " ++ show e ++ " - please report"
+
+mkFloat :: Const -> Double
+mkFloat (Num x) = toEnum x
+mkFloat (Ch c) = toEnum (fromEnum c)
+mkFloat (Bo False) = 0.0
+mkFloat (Bo True) = 1.0
+mkFloat (Re x) = x
+mkFloat (Str str) = read str
+mkFloat e 
+    = error $ "This can't happen: mkFloat " ++ show e ++ " - please report"
+
 -- Apply a function (non-recursively) to every sub expression,
 -- applying a different function to metavariables
 -- (I don't know if this is actually that useful, but it is used by the 
@@ -442,6 +482,7 @@ mapsubexpr f mf expr = app expr
 	app (AssignOp op a e) = AssignOp op (aapply a) (f e)
 	app (Seq a b) = Seq (f a) (f b)
 	app (Apply fn as) = Apply (f fn) (applys as)
+	app (ConApply fn as) = ConApply (f fn) (applys as)
 	app (Partial fn as i) = Partial (f fn) (applys as) i
 	app (Foreign ty n es) = Foreign ty n 
 			        (zip (applys (map fst es)) (map snd es))
@@ -456,6 +497,7 @@ mapsubexpr f mf expr = app expr
 	app (CmpExcept op x y) = CmpExcept op (f x) (f y)
 	app (RealInfix op x y) = RealInfix op (f x) (f y)
 	app (Append x y) = Append (f x) (f y)
+        app (AppendChain xs) = AppendChain (applys xs)
 	app (Unary op x) = Unary op (f x)
 	app (RealUnary op x) = RealUnary op (f x)
 	app (Coerce t1 t2 x) = Coerce t1 t2 (f x)
@@ -465,6 +507,7 @@ mapsubexpr f mf expr = app expr
 	app (Field e n i j) = Field (f e) n i j
 	app (ArrayInit as) = ArrayInit (applys as)
 	app (Annotation a e) = Annotation a (f e)
+        app (Length s) = Length (f s)
 	app x = x
 
         aapply (AIndex a e) = AIndex (aapply a) (f e)
@@ -477,6 +520,8 @@ mapsubexpr f mf expr = app expr
         altapp [] = []
 	altapp ((Alt i j es e):as) 
 	    = (Alt i j (applys es) (f e)):(altapp as)
+	altapp ((ArrayAlt es e):as) 
+	    = (ArrayAlt (applys es) (f e)):(altapp as)
 	altapp ((Default e):as) 
 	    = (Default (f e)):(altapp as)
 	altapp ((ConstAlt pt c e):as) 
@@ -496,6 +541,7 @@ foldsubexpr f com def expr = app expr
 	app (AssignOp op a e) = (aapply a) `com` (f e)
 	app (Seq a b) = (f a) `com` (f b)
 	app (Apply fn as) = (f fn) `com` (applys as)
+	app (ConApply fn as) = (f fn) `com` (applys as)
 	app (Partial fn as i) = (f fn) `com` (applys as)
 	app (Foreign ty n es) = applys (map fst es)
 	app (While e b) = (f e) `com` (f b)
@@ -531,6 +577,8 @@ foldsubexpr f com def expr = app expr
         altapp [] = def
 	altapp ((Alt i j es e):as) 
 	    = (applys es) `com` (f e) `com` (altapp as)
+	altapp ((ArrayAlt es e):as) 
+	    = (applys es) `com` (f e) `com` (altapp as)
 	altapp ((Default e):as) 
 	    = (f e) `com` (altapp as)
 	altapp ((ConstAlt pt c e):as) 
@@ -548,6 +596,7 @@ locsUsed (AssignOp _ a e) = alocsUsed a ++ locsUsed e
 locsUsed (AssignApp a e) = alocsUsed a ++ locsUsed e
 locsUsed (Seq x y) = locsUsed x ++ locsUsed y
 locsUsed (Apply f as) = locsUsed f ++ concat (map locsUsed as)
+locsUsed (ConApply f as) = locsUsed f ++ concat (map locsUsed as)
 locsUsed (Partial f as _) = locsUsed f ++ concat (map locsUsed as)
 locsUsed (Foreign _ _  as) = concat (map locsUsed (map fst as))
 locsUsed (While x y) = locsUsed x ++ locsUsed y
@@ -585,6 +634,7 @@ clocsUsed (Catch (Left (n,es)) h) = concat (map locsUsed es) ++ locsUsed h
 
 caseLocsUsed (Default e) = locsUsed e
 caseLocsUsed (Alt _ _ es e) = concat (map locsUsed es) ++ locsUsed e
+caseLocsUsed (ArrayAlt es e) = concat (map locsUsed es) ++ locsUsed e
 caseLocsUsed (ConstAlt _ _ e) = locsUsed e
 
 -- get a list of all variables which are modified (i.e. assigned to,
@@ -618,6 +668,7 @@ modVar (ArrayInit es) = concat (map modVar es)
 modVar (NewTryCatch e cs) = modVar e ++ concat (map cmodVar cs)
 -- This ought to check if the arg is a var arg.
 modVar (Apply _ args) = concat (map locsUsed args)
+modVar (ConApply _ args) = concat (map locsUsed args)
 -- May be over conservative, but unlikely to happen much in practice.
 modVar (Partial _ args _) = concat (map locsUsed args)
 modVar (Foreign _ _ args) = concat (map locsUsed (map fst args))
@@ -631,6 +682,7 @@ isArg (Loc i) = [i]
 isArg _ = []
 
 inAlt (Alt _ _ args e) = modVar e ++ concat (map isArg args)
+inAlt (ArrayAlt args e) = modVar e ++ concat (map isArg args)
 inAlt (ConstAlt _ _ e) = modVar e
 inAlt (Default e) = modVar e
 
@@ -653,11 +705,31 @@ sizemangle = mangling (Fn [] [Array (TyVar (UN "a"))] (Prim Number))
 missingCase = NS (UN "Builtins") (UN "Missing_Case")
 missingCaseMangling = mangling (Fn [] [] (Prim Exception));
 
+pmAssignFail = NS (UN "Builtins") (UN "Pattern_Matching_Assignment_Failure")
+pmAssignFailMangling = mangling (Fn [] [] (Prim Exception));
+
 exitfun = (NS (UN "Builtins") (UN "exit"))
 exitmangle = mangling (Fn [] [(Prim Number)] (Prim Void))
 
+pushfun = (NS (UN "Array") (UN "push"))
+
 putstrlnfun = (NS (UN "Prelude") (UN "putStrLn"))
 putstrlnmangle = mangling (Fn [] [(Prim StringType)] (Prim Void))
 
 backtracefun = (NS (UN "Builtins") (UN "exceptionBacktrace"))
-backtracemangle = mangling (Fn [] [(Prim Exception)] (Prim Void))
\ No newline at end of file
+backtracemangle = mangling (Fn [] [(Prim Exception)] (Prim Void))
+
+tappFun = (NS (UN "Multicore") (UN "tapply"))
+tappMangle i = mangling (Fn [] (tappArgs 0 i) (TyVar (UN "a")))
+   where tappArgs i num 
+             | i == num = []
+             | otherwise = (TyVar (UN ("a"++show i))):(tappArgs (i+1) num)
+
+texecFun = (NS (UN "Multicore") (UN "texec"))
+texecMangle i = mangling (Fn [] (tappArgs 0 i) (Prim Void))
+   where tappArgs i num 
+             | i == num = []
+             | otherwise = (TyVar (UN ("a"++show i))):(tappArgs (i+1) num)
+
+dumpFun = (NS (UN "Reflect") (UN "dump"))
+dumpMangle = mangling (Fn [] [(TyVar (UN "a"))] (Prim Void))
\ No newline at end of file
