@@ -49,6 +49,8 @@ extern "C" {
     int pg_numcols(void* res);
 /// Close the connection
     void pg_close(void* conn);
+    void* do_pg_prepare(wchar_t* query);
+    void* pg_execp(void* vmptr,void* conn,void* queryptr, KayaValue err, KArray params);
 }
 
 #endif
