@@ -15,6 +15,17 @@ public:
     int ok;
 };
 
+class SQLiteStatement : public gc_cleanup {
+public:
+    sqlite3_stmt* stmt;
+    SQLiteStatement(sqlite3_stmt* s):stmt(s) {}
+    ~SQLiteStatement() {
+      if (stmt != NULL) {
+        sqlite3_finalize(stmt);
+      }
+    }
+};
+
 class SQLiteResult : public gc {
 public:
     KArray res_table;
@@ -31,6 +42,9 @@ extern "C" {
     void sqlite_close(void* con);
     void* sqlite_exec(void* con, wchar_t* query);
 
+    void* sqlite_execp(void* stptr, KArray params);
+    void* sqlite_prepare(void* vmptr, void* con, wchar_t* query);
+
     KArray sql_getcolnames(void* res);
     KArray sql_getrestable(void* res);
     int sql_numrows(void* res);
