@@ -14,13 +14,13 @@ using namespace std;
 #include "VMState.h"
 #include "stdfuns.h"
 
-Closure::Closure(VMState* vm, func fn, int args, int arity, bool stack):m_fn(fn),m_arity(arity)
+Closure::Closure(VMState* vm, func fn, kint args, kint arity, bool stack):m_fn(fn),m_arity(arity)
 {
 //    cout << "Making closure " << (void*)(m_fn) << endl;
     m_args=(Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*arity);
     m_numargs=args;
     if (stack) {
-	for(int i=0;i<args;++i) {
+	for(kint i=0;i<args;++i) {
 //	cout << "Arg" << endl;
 /// Create a *new* reference, or things break in confusing ways.
 	  m_args[i]=vm->doPop()->clone();
@@ -29,10 +29,29 @@ Closure::Closure(VMState* vm, func fn, int args, int arity, bool stack):m_fn(fn)
     }
 }
 
-void Closure::addArgs(VMState* vm, int added)
+Closure::Closure(Closure& x) {
+    // Shallow copy everything, we just need a new argument list.
+    m_numargs = x.m_numargs;
+    m_arity = x.m_arity;
+    m_fn = x.m_fn;
+    m_args=(Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*m_arity);
+    for(kint i=0;i<m_numargs;++i) {
+	// Make it a new pointer, so that we don't modify inside it.
+	m_args[i]=x.m_args[i]->clone();
+    }
+}
+
+void Closure::addArgs(VMState* vm, kint added)
 {
-    int newnum = m_numargs+added;
-    int i;
+    kint newnum = m_numargs+added;
+    kint i;
+
+    // Better make sure there's enough room! (it really should have been
+    // allocated, but there may be circumstances where it hasn't, if someone
+    // is abusing the type system...)
+    if (newnum>m_arity) {
+	m_args = (Value**)GC_REALLOC(m_args, newnum);
+    }
 
     for(i=m_numargs; i<newnum; ++i) {
         m_args[i]=vm->doPop()->clone();
@@ -47,7 +66,7 @@ Closure::~Closure()
 
 void Closure::run(VMState* vm)
 {
-    for(int i=m_numargs;i>0;i--) {
+    for(kint i=m_numargs;i>0;i--) {
 	vm->push(m_args[i-1]);
     }
     m_fn(vm);
@@ -58,7 +77,7 @@ bool Closure::eq(Closure* c, map<Value*, Value*>& done)
     if (getFnID()!=c->getFnID()) { return false; }
     if (m_numargs!=c->m_numargs) { return false; }
 
-    for(int i=0;i<m_numargs;++i)
+    for(kint i=0;i<m_numargs;++i)
     {
 	if (!funtable_eq_aux(m_args[i],c->m_args[i],done)) { return false; }
     }
@@ -69,7 +88,7 @@ bool Closure::eq(Closure* c, map<Value*, Value*>& done)
 Closure* Closure::copy(map<Value*, Value*>& done)
 {
     Closure* c = new Closure(NULL,m_fn,m_numargs,m_arity,false);
-    for(int i=0;i<m_numargs;++i) {
+    for(kint i=0;i<m_numargs;++i) {
 	Value* copied;
 	Value* x = m_args[i];
 	// if x is already copied, return the copied value
