@@ -11,7 +11,7 @@
 
 #ifndef WIN32
 #include <pthread.h>
-#define GC_PTHREADS
+#define GC_THREADS
 #else
 #define GC_WIN32_THREADS
 #endif
@@ -24,7 +24,8 @@ class Closure : public gc_cleanup {
 public:
     // Create a closure from a function, which will have <args> as its locals,
     // and space for <arity> arguments.
-    Closure(VMState* vm, func fn,int args, int arity, bool stack=true);
+    Closure(VMState* vm, func fn,kint args, kint arity, bool stack=true);
+    Closure(Closure& x);
     ~Closure();
 
     // Run the closure. Pushes m_args onto the stack, then runs the function
@@ -32,12 +33,12 @@ public:
     void run(VMState* vm);
 
     func getfn() { return m_fn; }
-    int getFnID() { return ::getFnID(m_fn); }
-    int getNumArgs() { return m_numargs; }
+    kint getFnID() { return ::getFnID(m_fn); }
+    kint getNumArgs() { return m_numargs; }
     Value** getArgs() { return m_args; }
 
-    void setArg(int i,Value* v) { m_args[i]=v; }
-    void addArgs(VMState* vm, int i);
+    void setArg(kint i,Value* v) { m_args[i]=v; }
+    void addArgs(VMState* vm, kint i);
 
     bool eq(Closure* c,map<Value*,Value*>& done);
     Closure* copy(map<Value*, Value*>& done);
@@ -45,8 +46,8 @@ public:
 private:
     func m_fn;
     Value** m_args;
-    int m_numargs;
-    int m_arity;
+    kint m_numargs;
+    kint m_arity;
 };
 
 #endif
