@@ -17,15 +17,15 @@
 
 /// function map is global to all threads.
 func* m_funmap[1];
-int m_funmapsize;
-int m_funmaphash;
+kint m_funmapsize;
+kint m_funmaphash;
 
 // Defined by compiling Builtins.k, so obviously any binary of a kaya
 // program needs to be linked to at least Builtins.o!
-#define INTERNAL_ERROR _ns__D_Builtins__D_InternalError
+#define INTERNAL_ERROR K_ns__D_Builtins__D_InternalError
 extern char* INTERNAL_ERROR;
 
-void initFunMap(int sz, int fmhash)
+void initFunMap(kint sz, kint fmhash)
 {
     func* funcs = (func*)malloc(sizeof(func)*sz);
     m_funmap[0] = funcs;
@@ -33,42 +33,47 @@ void initFunMap(int sz, int fmhash)
     m_funmaphash = fmhash;
 }
 
-void addToFunMap(int id, func fn)
+void addToFunMap(kint id, func fn)
 {
   (m_funmap[0])[id] = fn;
 }
 
-func getFn(int id)
+func getFn(kint id)
 {
-  // should add bounds checking here
+  if (id > 0 && id < m_funmapsize) {
     return (m_funmap[0])[id];
+  } else {
+    // FIXME: do something sensible with this result in the calling context
+    return NULL;
+  }
 }
 
-int getFnID(func fn)
+kint getFnID(func fn)
 {
-    int i;
+    kint i;
     for (i=1;i<m_funmapsize;++i) {
 	if (m_funmap[0][i] == fn) { return i; }
     }
     return -1;
 }
 
-int getFnHash()
+kint getFnHash()
 {
     return m_funmaphash;
 }
 
 VMState::VMState(bool panic):m_return(NULL,KVT_NULL) {
-    m_stackalloc=100;
-    m_globalloc=100;
+    m_stackalloc=128;
+    m_callstackalloc = 1024;
+    m_globalloc=128;
     if (!panic) {
 	m_valstack = (Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*m_stackalloc);
 	// Make sure stack is clear.
 	memset(m_valstack,0,sizeof(Value*)*m_stackalloc);
 	m_maxmem = 0;
-	m_backtrace = (CallStackEntry**)GC_MALLOC_UNCOLLECTABLE(sizeof(CallStackEntry*)*m_stackalloc*10);
+	m_backtrace = (CallStackEntry**)GC_MALLOC_UNCOLLECTABLE(sizeof(CallStackEntry*)*m_callstackalloc);
 	// Allocate space all at once (max out at <m_stackalloc*10>)
-	for(int i=0;i<m_stackalloc*10;++i) {
+	for(int i=0;i<m_callstackalloc;++i) {
 	    m_backtrace[i]=new CallStackEntry;
 	}
     }
@@ -128,45 +133,43 @@ void VMState::push4(Value* val, Value* val2, Value* val3, Value* val4)
     *(m_stackptr++)=val4;
 }
 
-int VMState::tag()
+kint VMState::tag()
 {
-#ifndef NOCHECK
-    if ((*(m_stackptr-1))->getType()!=KVT_UNION) {
-	kaya_rtsError(GETTING_TAG_FROM_NON_UNION);
-    }
-#endif
-    return (*(m_stackptr-1))->getTag();
+  valtype t = (*(m_stackptr-1))->getType();
+  if (t >= KVT_UNION) {
+    return t-KVT_UNION;
+  } else if (t == KVT_CUNION) {
+    return (*(m_stackptr-1))->getInt();
+  } else {
+    // no extra cost to doing this check now...
+    kaya_rtsError(GETTING_TAG_FROM_NON_UNION);
+    // doesn't matter what we return, but it keeps the compiler happy
+    return 0;
+  }
 }
 
 void VMState::doAppend()
 {
     Value* str2=doPop();
     Value* str1=doPop();
-    String* str=new String(str1->getString()->getVal());
+    String* str=NEWSTRING(str1->getString()->getVal());
     str->append(str2->getString());
-    push(new Value((void*)str,KVT_STRING));
+    push(MKVAL((void*)str,KVT_STRING));
 }
 
-void VMState::doAppendChar(wchar_t x)
+void VMState::doAppendChar(const wchar_t x)
 {
     // append x onto top stack item
     (*(m_stackptr-1))->getString()->append(x);
 }
 
-void VMState::doAppendStr(wchar_t* x)
+void VMState::doAppendStr(const wchar_t* x)
 {
     // append x onto top stack item
     (*(m_stackptr-1))->getString()->append(x);
 
 }
 
-void VMState::doAppendStr(char* x)
-{
-    // append x onto top stack item
-//    (*(m_stackptr-1))->getString()->append(x); unsound!
-  VMState::doAppendStr(strtowc(x));
-}
-
 void VMState::doEqExcept(bool inv)
 {
     Value* e2=doPop();
@@ -177,47 +180,50 @@ void VMState::doEqExcept(bool inv)
     else {  push(zero); }
 }
 
-void VMState::doEqString(bool inv)
+void VMState::doEqString()
 {
     Value* e2=doPop();
     Value* e1=doPop();
     bool eq = (e2->getString()->eq(e1->getString()));
-    if (inv) { eq = !eq; }
     if (eq) {  push(one); }
     else {  push(zero); }
 }
 
-void VMState::doEqString(bool inv, wchar_t* str)
+void VMState::doNeString()
+{
+    Value* e2=doPop();
+    Value* e1=doPop();
+    bool eq = (e2->getString()->eq(e1->getString()));
+    if (eq) {  push(zero); }
+    else {  push(one); }
+}
+
+void VMState::doEqString(const wchar_t* str)
 {
     Value* e1=doPop();
     bool eq = (e1->getString()->eq(str));
-    if (inv) { eq = !eq; }
     if (eq) {  push(one); }
     else {  push(zero); }
 }
 
-void VMState::doEqString(bool inv, char* str)
+void VMState::doNeString(const wchar_t* str)
 {
-  return VMState::doEqString(inv,strtowc(str));
+    Value* e1=doPop();
+    bool eq = (e1->getString()->eq(str));
+    if (eq) {  push(zero); }
+    else {  push(one); }
 }
 
-
-bool VMState::doTopStrEq(wchar_t* str)
+bool VMState::doTopStrEq(const wchar_t* str)
 {
     wchar_t* topstr = (*(m_stackptr-1))->getString()->getVal();
     return (!wcscmp(topstr,str));
 }
 
-bool VMState::doTopStrEq(char* str)
-{
-  return VMState::doTopStrEq(strtowc(str));
-}
-
-
 void VMState::int2str()
 {
     // Need to do this on a copy or we break types.
-    Value* x = new Value(NULL,KVT_NULL);
+    Value* x = MKVAL(NULL,KVT_NULL);
     x->setInt(doPop()->getInt());
     x->int2str();
     push(x);
@@ -226,7 +232,7 @@ void VMState::int2str()
 void VMState::real2str()
 {
     // Need to do this on a copy or we break types.
-    Value* x = new Value(NULL,KVT_NULL);
+    Value* x = MKVAL(NULL,KVT_NULL);
     x->setReal(doPop()->getReal());
     x->real2str();
     push(x);
@@ -235,7 +241,7 @@ void VMState::real2str()
 void VMState::str2int()
 {
     // Need to do this on a copy or we break types.
-    Value* x = new Value(doPop()->getRaw(),KVT_INT);
+    Value* x = MKVAL(doPop()->getRaw(),KVT_INT);
     x->str2int();
     push(x);
 }
@@ -243,7 +249,7 @@ void VMState::str2int()
 void VMState::str2real()
 {
     // Need to do this on a copy or we break types.
-    Value* x = new Value(doPop()->getRaw(),KVT_REAL);
+    Value* x = MKVAL(doPop()->getRaw(),KVT_REAL);
     x->str2real();
     push(x);
 }
@@ -251,7 +257,7 @@ void VMState::str2real()
 void VMState::chr2str()
 {
     // Need to do this on a copy or we break types.
-    Value* x = new Value(doPop()->getRaw(),KVT_STRING);
+    Value* x = MKVAL(doPop()->getRaw(),KVT_STRING);
     x->chr2str();
     push(x);
 }
@@ -259,7 +265,7 @@ void VMState::chr2str()
 void VMState::bool2str()
 {
     // Need to do this on a copy or we break types.
-    Value* x = new Value(doPop()->getRaw(),KVT_STRING);
+    Value* x = MKVAL(doPop()->getRaw(),KVT_STRING);
     x->bool2str();
     push(x);
 }
@@ -273,7 +279,7 @@ void VMState::str2chr()
 void VMState::real2int()
 {
     // Need to do this on a copy or we break types.
-    Value* x = new Value(NULL,KVT_NULL);
+    Value* x = MKVAL(NULL,KVT_NULL);
     x->setReal(doPop()->getReal());
     x->real2int();
     push(x);
@@ -282,13 +288,23 @@ void VMState::real2int()
 void VMState::int2real()
 {
     // Need to do this on a copy or we break types.
-    Value* x = new Value(NULL,KVT_NULL);
+    Value* x = MKVAL(NULL,KVT_NULL);
     x->setInt(doPop()->getInt());
     x->int2real();
     push(x);
 }
 
-void VMState::mkArray(int i)
+double VMState::topint2real()
+{
+    // Need to do this on a copy or we break types.
+    Value xv(NULL,KVT_NULL);
+    Value* x = &xv;
+    x->setInt(doPop()->getInt());
+    x->int2real();
+    return x->getReal();
+}
+
+void VMState::mkArray(kint i)
 {
     Array* array = new Array(i*2);
     // leave a bit of room for expansion (costs a little memory)
@@ -298,29 +314,29 @@ void VMState::mkArray(int i)
 	i--;
     }
 
-    push(new Value(array,KVT_ARRAY));
+    push(MKVAL(array,KVT_ARRAY));
 }
 
 void VMState::goToIndex()
 {
     --m_stackptr;
-    int idx = (*m_stackptr)->getInt();
+    kint idx = (*m_stackptr)->getInt();
     (*(m_stackptr-1)) = (*(m_stackptr-1))->lookup(this,idx);
 }
 
-void VMState::pushglobal(Value**& globtable, int i)
+void VMState::pushglobal(Value**& globtable, kint j)
 {
     if (globtable == NULL) {
 	globtable = (Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*m_globalloc);
-	for(int i=0;i<m_globalloc;++i) { //TMP HACK
-	    globtable[i]=new Value(0,KVT_INT);
+	for(kint i=0;i<m_globalloc;++i) { //TMP HACK
+	    globtable[i]=MKVAL(0,KVT_INT);
 	}
     }
 
-    push(globtable[i]);
+    push(globtable[j]);
 }
 
-void VMState::createglobal(wchar_t* modid, int i)
+void VMState::createglobal(wchar_t* modid, kint i)
 {
     cerr << "Do not run this function again" << endl;
     exit(1);
@@ -341,31 +357,25 @@ void VMState::newjmp_buf()
 //    cout << "Inside " << except_stack.size() << " tries" << endl;
 }
 
-void VMState::kaya_throw(wchar_t* msg,int code)
+void VMState::kaya_throw(wchar_t* msg,kint code)
 {
     push(mkstr(msg));
     push(mkint((void*)code));
-    push(new Value((void*)(new Exception(this)), KVT_EXCEPTION));
+    push(MKVAL((void*)(new Exception(this)), KVT_EXCEPTION));
     throw_ex();
 }
 
-void VMState::kaya_throw(char* msg,int code)
-{
-    // CIM: call the wchar_t version to make it easier to put strings in code
-    kaya_throw(strtowc(msg),code);
-}
-
-void VMState::kaya_internalError(int code)
+void VMState::kaya_internalError(kint code)
 {
     push(mkint((void*)code));
-    push(new Value((void*)(new Exception(this, INTERNAL_ERROR, 1)), 
+    push(MKVAL((void*)(new Exception(this, INTERNAL_ERROR, 1)), 
 		   KVT_EXCEPTION));
     throw_ex();
 }
 
-void VMState::kaya_rtsError(char* exceptionID)
+void VMState::kaya_rtsError(const char* exceptionID)
 {
-    push(new Value((void*)(new Exception(this, exceptionID, 0)), 
+    push(MKVAL((void*)(new Exception(this, exceptionID, 0)), 
 		   KVT_EXCEPTION));
     throw_ex();
 }
@@ -395,7 +405,7 @@ void VMState::restore()
 
 void VMState::getindex()
 {
-    int idx = doPop()->getInt();
+    kint idx = doPop()->getInt();
     Value* array = doPop();
     Value* el = array->lookup(this,idx);
     push(el);
@@ -403,16 +413,12 @@ void VMState::getindex()
 
 Value* VMState::mkstr(wchar_t* str)
 {
-    return new Value((void*)(new String(str)),KVT_STRING);
+    return MKVAL((void*)(NEWSTRING(str)),KVT_STRING);
 }
 
 Value* VMState::mkint(void* i)
 {
-    return new Value(i,KVT_INT);
-}
-
-void VMState::lineno(const char* src, int ln) {
-  VMState::lineno(strtowc(src),ln);
+    return MKVAL(i,KVT_INT);
 }
 
 void VMState::lineno(const wchar_t* src, int ln)
@@ -425,11 +431,7 @@ void VMState::lineno(const wchar_t* src, int ln)
 #endif
 }
 
-void VMState::pushBT(char* fn, char* mod, int ln) {
-  VMState::pushBT(strtowc(fn),strtowc(mod),ln);
-}
-
-void VMState::pushBT(wchar_t* fn, wchar_t* mod, int ln)
+void VMState::pushBT(const wchar_t* fn, const wchar_t* mod, int ln)
 {
 #ifndef NOCHECK
     CallStackEntry* c = *m_btptr;
@@ -441,8 +443,9 @@ void VMState::pushBT(wchar_t* fn, wchar_t* mod, int ln)
     c->call_line = m_lineno;
 
     ++m_btptr;
-    if ((m_btptr-m_backtrace)>=m_stackalloc*10) {
-	kaya_rtsError(CALL_STACK_OVERFLOW);
+    if ((m_btptr-m_backtrace)>=m_callstackalloc) {
+      realloc_callstack();
+      //      kaya_rtsError(CALL_STACK_OVERFLOW);
     }
 #endif
 }
@@ -475,3 +478,37 @@ void VMState::writestdout()
     fsync(1);
 #endif
 }
+
+void VMState::realloc_stack() {
+  //  cerr << "realloc:" << m_stackalloc << endl;
+  Value** newstack = (Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*(m_stackalloc<<3));
+  memset(newstack,0,sizeof(Value*)*(m_stackalloc<<3));
+  for (kint i=0;i<m_stackalloc;i++) {
+    newstack[i] = m_valstack[i];
+  }
+  kint h = m_stackptr-m_valstack;
+  Value** oldstack = m_valstack;
+  m_valstack = newstack;
+  GC_FREE(oldstack);
+  m_stackptr = m_valstack+h;
+  m_stackalloc = m_stackalloc<<3;
+  // generally this only happens with heavily recursive functions
+}
+
+void VMState::realloc_callstack() {
+  //  cerr << "realloc call:" << m_callstackalloc << endl;
+  CallStackEntry** newstack = (CallStackEntry**)GC_MALLOC_UNCOLLECTABLE(sizeof(CallStackEntry*)*(m_callstackalloc<<3));
+  for (kint i=0;i<m_callstackalloc;i++) {
+    newstack[i] = m_backtrace[i];
+  }
+  for (kint i=m_callstackalloc;i<(m_callstackalloc<<3);i++) {
+    newstack[i]=new CallStackEntry;
+  }
+  kint h = m_btptr-m_backtrace;
+  CallStackEntry** oldstack = m_backtrace;
+  m_backtrace = newstack;
+  GC_FREE(oldstack);
+  m_btptr = m_backtrace+h;
+  m_callstackalloc = m_callstackalloc<<3;
+  // generally this only happens with heavily recursive functions
+}
