@@ -27,13 +27,15 @@
 #include <unistd.h>
 #include <cstdlib>
 #include <vector>
+#include <stddef.h>
+#include <inttypes.h>
 //#include <malloc.h>
 
 // CIM: pthread.h is outside the thread definitions since we need it anyway
 // for the gmtime_r functions.
 #include <pthread.h>
 #ifndef WIN32
-#define GC_PTHREADS
+#define GC_THREADS
 #else
 #define GC_WIN32_THREADS
 #endif
@@ -52,14 +54,14 @@
 KayaArray args;
 ArgMap httpargs;
 
-wchar_t* inttostr(int s, int base)
+wchar_t* inttostr(kint s, kint base)
 {
   // CIM: I'm sure we can make this smaller
     int sz = 255;
     wchar_t* buf=(wchar_t*)GC_MALLOC_ATOMIC(sz*sizeof(wchar_t));
     switch(base) {
     case 10:
-	SWPRINTF(buf,sz,L"%d",s);
+	SWPRINTF(buf,sz,DIGITFORMAT,s);
 	break;
     case 16:
 	SWPRINTF(buf,sz,L"%x",s);
@@ -68,18 +70,22 @@ wchar_t* inttostr(int s, int base)
 	SWPRINTF(buf,sz,L"%o",s);
 	break;
     default:
-	SWPRINTF(buf,sz,L"%d",s); // Can't do weird bases yet.
+	SWPRINTF(buf,sz,DIGITFORMAT,s); // Can't do weird bases yet.
 	break;
     }
     return buf;
 }
 
-int strtoint(wchar_t* str, int base)
+kint strtoint(wchar_t* str, kint base)
 {
+#ifdef KAYA_SHORT_INTS
+    return wcstoimax(str,NULL,base);
+#else
     return wcstol(str,NULL,base);
+#endif
 }
 
-wchar_t* getsubstr(wchar_t* x,int i,int len)
+wchar_t* getsubstr(wchar_t* x,kint i,kint len)
 {
     wchar_t* n = (wchar_t*)GC_MALLOC_ATOMIC((len+1)*sizeof(wchar_t));
     if (len > 0) {
@@ -89,19 +95,19 @@ wchar_t* getsubstr(wchar_t* x,int i,int len)
     return n;
 }
 
-void str_offset(Value* str, int inc)
+void str_offset(Value* str, kint inc)
 {
     str->getString()->offset(inc);
 }
 
-void str_chop(Value* str, int inc)
+void str_chop(Value* str, kint inc)
 {
     str->getString()->chop(inc);
 }
 
-wchar_t* getstrend(wchar_t* x,int i)
+wchar_t* getstrend(wchar_t* x,kint i)
 {
-    int max = wcslen(x);
+    kint max = wcslen(x);
     wchar_t* n = (wchar_t*)GC_MALLOC_ATOMIC((max+1-i)*sizeof(wchar_t));
     if (max-i > 0) {
       wcsncpy(n,x+i,max-i);
@@ -109,18 +115,19 @@ wchar_t* getstrend(wchar_t* x,int i)
     n[max-i] = '\0';
     return n;
 }
-wchar_t getIndex(wchar_t* str,int i)
+wchar_t getIndex(wchar_t* str,kint i)
 {
     return str[i];
 }
 
-void setIndex(wchar_t* str, wchar_t c,int i)
+void setIndex(wchar_t* str, wchar_t c,kint i)
 {
     str[i] = c;
 }
 
-wchar_t* getLine(FILE* f)
+wchar_t* getLine(void* fptr)
 {
+    FILE* f = (FILE*)fptr;
     unsigned size = LINE_CHUNK; // #define at top of file
     // buffer to read into
     char* buf=(char*)GC_MALLOC_ATOMIC(LINE_CHUNK*sizeof(char)+1); 
@@ -168,8 +175,21 @@ wchar_t* getLine(FILE* f)
     return emptyString(); // Failed, although we shouldn't get here
 }
 
-wchar_t* getString(FILE* f)
+wchar_t do_fgetc(void* fptr) {
+  return (wchar_t)fgetc((FILE*)fptr);
+}
+
+void do_fputc(kint ch, void* fptr) {
+  fputc(ch,(FILE*)fptr);
+}
+
+void* do_tmpFile() {
+  return (void*)tmpfile();
+}
+
+wchar_t* getString(void* fptr)
 {
+    FILE* f = (FILE*)fptr;
     unsigned size = LINE_CHUNK; // #define at top of file
     // buffer to read into
     char* buf=(char*)GC_MALLOC_ATOMIC(LINE_CHUNK*sizeof(char)+1); 
@@ -197,37 +217,51 @@ wchar_t* getString(FILE* f)
     return emptyString(); // Failed, although we shouldn't get here
 }
 
-void putLine(FILE* f, wchar_t* c)
+void putLine(void* fptr, wchar_t* c, kint len)
 {
-    fputs(wctostr(c),f);
+    FILE* f = (FILE*)fptr;
+    fputs(wcntostr(c,len),f);
 }
 
-void putString(FILE* f, wchar_t* c)
+void putString(void* fptr, wchar_t* c, kint len)
 {
-    fputs(wctostr(c),f);
+    FILE* f = (FILE*)fptr;
+    fputs(wcntostr(c,len),f);
     fputc('\0',f);
 }
 
-void do_fseek(FILE* f, kint p) {
-    fseek(f,(long)p,SEEK_SET);
+void do_fseek(void* f, kint p) {
+    fseek((FILE*)f,(long)p,SEEK_SET);
 }
 
-kint do_ftell(FILE* f) {
-  return KINT(ftell(f));
+kint do_ftell(void* f) {
+  return KINT(ftell((FILE*)f));
 }
 
-void putStr(wchar_t* rawc)
+bool do_feof(void* f) {
+  return feof((FILE*)f);
+}
+
+kint do_fflush(void* f) {
+  return fflush((FILE*)f);
+}
+
+void do_fclose(void* f) {
+  fclose((FILE*)f);
+}
+
+
+void putStr(wchar_t* rawc, kint rlen)
 {
-    unsigned int len = 1+wcslen(rawc);
-    int i = sizeof(char)*(len)*4;
+    kint i = sizeof(char)*(++rlen)*4;
     char c[i];
-    mkUTF8(rawc,c,len);
+    mkUTF8(rawc,c,rlen);
     cout << c;
 }
 
 // Check if the top bit is set in a string (i.e., whether it is ASCII or
 // includes UTF-8 characters)
-int topSet(const char* str) {
+kint topSet(const char* str) {
     const char* sptr = str;
     for(;(*sptr)!=0;++sptr) {
 	if (((*sptr)&128)!=0) {
@@ -237,14 +271,20 @@ int topSet(const char* str) {
     return 0;
 }
 
-int checkUTF8(wchar_t* str)
+kint checkUTF8(wchar_t* str)
 {
-    return(topSet(CSTRING(str)));
+    wchar_t* sptr = str;
+    for(;(*sptr)!=0;++sptr) {
+	if ((*sptr)>127) {
+	    return 1;
+	}
+    }
+    return 0;
 }
 
 void printArray(KayaArray foo)
 {
-    for(int i=0;i<foo->size();++i) {
+    for(unsigned i=0;i<foo->size();++i) {
 	cout << wctostr(foo->lookup(i)->getString()->getVal()) << endl;
     }
 }
@@ -270,35 +310,40 @@ KayaValue shiftArray(KayaArray foo)
     return foo->shift();
 }
 
-KayaArray createArray(int size)
+void unshiftArray(KayaValue val, KayaArray foo)
+{
+    return foo->unshift(val);
+}
+
+KayaArray createArray(kint size)
 {
     return newKayaArray(size);
 }
 
-void resizeArray(KayaArray foo, int size)
+void resizeArray(KayaArray foo, kint size)
 {
     foo->resize(size);
 }
 
-int arraySize(KayaArray foo)
+kint arraySize(KayaArray foo)
 {
     return foo->size();
 }
 
 
-FILE* getstdin()
+void* getstdin()
 {
-    return stdin;
+    return (void*)stdin;
 }
 
-FILE* getstdout()
+void* getstdout()
 {
-    return stdout;
+    return (void*)stdout;
 }
 
-FILE* getstderr()
+void* getstderr()
 {
-    return stderr;
+    return (void*)stderr;
 }
 
 void setStdErr(FILE* f)
@@ -308,16 +353,16 @@ void setStdErr(FILE* f)
 #endif
 }
 
-bool validFile(FILE* f)
+bool validFile(void* f)
 {
-    return f!=NULL;
+    return (FILE*)f!=NULL;
 }
 
 void storeArgs(int argc,char* argv[])
 {
     args = new Array(argc);
     for (int i=0;i<argc;++i) {
-	KayaValue arg = new Value((void*)(new String(strtowc(argv[i]))),KVT_STRING);
+	KayaValue arg = MKVAL((void*)(NEWSTRING(strtowc(argv[i]))),KVT_STRING);
 	args->push_back(arg);
     }
 }
@@ -327,9 +372,9 @@ KayaArray getArgs()
     return args;
 }
 
-int gettime()
+kint gettime()
 {
-    return (int)(time(NULL));
+    return (kint)(time(NULL));
 }
 
 kint dogettimeofday() {
@@ -340,7 +385,7 @@ kint dogettimeofday() {
 
 #ifdef WIN32
 // taken from libiberty (LGPL) in the GCC libraries
-int gettimeofday (struct timeval *tp, void *tz)
+kint gettimeofday (struct timeval *tp, void *tz)
 {     
   /* Offset between 1/1/1601 and 1/1/1970 in 100 nanosec units */
   static const unsigned long long W32_FT_OFFSET = 116444736000000000ULL;
@@ -369,49 +414,49 @@ int gettimeofday (struct timeval *tp, void *tz)
 }
 #endif
 
-KayaValue dogmtime(int secs)
+KayaValue dogmtime(kint secs)
 {
-    Union* tu = new Union(NULL,0,9,false);
+    Union* tu = new Union(NULL,9,false);
     struct tm ts;
     time_t t = (time_t)secs;
     gmtime_r(&t,&ts);
-    Union* mon = new Union(NULL,ts.tm_mon,0,false);
-    Union* wday = new Union(NULL,ts.tm_wday,0,false);
+    //    Union* mon = new Union(NULL,ts.tm_mon,0,false);
+    //    Union* wday = new Union(NULL,ts.tm_wday,0,false);
 
     tu->args[0]=mkint((void*)ts.tm_sec);
     tu->args[1]=mkint((void*)ts.tm_min);
     tu->args[2]=mkint((void*)ts.tm_hour);
     tu->args[3]=mkint((void*)ts.tm_mday);
-    tu->args[4]=MKUNION(mon);
+    tu->args[4]=MKVAL((void*)ts.tm_mon,KVT_CUNION);
     tu->args[5]=mkint((void*)(ts.tm_year+1900));
-    tu->args[6]=MKUNION(wday);
+    tu->args[6]=MKVAL((void*)ts.tm_wday,KVT_CUNION);
     tu->args[7]=mkint((void*)ts.tm_yday);
     tu->args[8]=mkint((void*)ts.tm_isdst);
-    return new Value((void*)tu,KVT_UNION);
+    return MKVAL((void*)tu,KVT_UNION);
 }
 
-KayaValue dolocaltime(int secs)
+KayaValue dolocaltime(kint secs)
 {
-    Union* tu = new Union(NULL,0,9,false);
+    Union* tu = new Union(NULL,9,false);
     struct tm ts;
     time_t t = (time_t)secs;
     localtime_r(&t,&ts);
-    Union* mon = new Union(NULL,ts.tm_mon,0,false);
-    Union* wday = new Union(NULL,ts.tm_wday,0,false);
+    //    Union* mon = new Union(NULL,ts.tm_mon,0,false);
+    //    Union* wday = new Union(NULL,ts.tm_wday,0,false);
 
     tu->args[0]=mkint((void*)ts.tm_sec);
     tu->args[1]=mkint((void*)ts.tm_min);
     tu->args[2]=mkint((void*)ts.tm_hour);
     tu->args[3]=mkint((void*)ts.tm_mday);
-    tu->args[4]=MKUNION(mon);
+    tu->args[4]=MKVAL((void*)ts.tm_mon,KVT_CUNION);
     tu->args[5]=mkint((void*)(ts.tm_year+1900));
-    tu->args[6]=MKUNION(wday);
+    tu->args[6]=MKVAL((void*)ts.tm_wday,KVT_CUNION);
     tu->args[7]=mkint((void*)ts.tm_yday);
     tu->args[8]=mkint((void*)ts.tm_isdst);
-    return new Value((void*)tu,KVT_UNION);
+    return MKVAL((void*)tu,KVT_UNION);
 }
 
-int domktime(KayaValue time)
+kint domktime(KayaValue time)
 {
     struct tm t;
     Union* tu = time->getUnion();
@@ -422,24 +467,24 @@ int domktime(KayaValue time)
     t.tm_year = (tu->args[5]->getInt())-1900;
     t.tm_yday = tu->args[7]->getInt();
     t.tm_isdst = tu->args[8]->getInt();
-    t.tm_mon = U_TAG(tu->args[4]->getUnion());
-    t.tm_wday = U_TAG(tu->args[6]->getUnion());
+    t.tm_mon = tu->args[4]->getType();
+    t.tm_wday = tu->args[6]->getType();
 
-    int x = (int)mktime(&t);
+    kint x = (kint)mktime(&t);
     return x;
 }
 
-int getclock()
+kint getclock()
 {
-    return (int)(clock());
+    return (kint)(clock());
 }
 #ifndef WIN32
-int runProgram(wchar_t* prog, KayaArray args)
+kint runProgram(wchar_t* prog, KayaArray args)
 {
     char** argv = new char*[args->size()+2];
 
     argv[0]=wctostr(prog);
-    for(int i=0;i<args->size();++i) {
+    for(unsigned i=0;i<args->size();++i) {
 	argv[i+1]=wctostr(args->lookup(i)->getString()->getVal());
     }
     argv[args->size()+1]=NULL;
@@ -457,9 +502,17 @@ int runProgram(wchar_t* prog, KayaArray args)
     return status;
 }
 
-int dofork()
+kint dofork()
 {
-    return (int)fork();
+    kint i = (kint)fork();
+    if (i<0) {
+	if (errno == ENOMEM) {
+	    return -1;
+	} else if (errno == EAGAIN) {
+	    return -2;
+	}
+    }
+    return i;
 }
 
 void reap()
@@ -467,39 +520,55 @@ void reap()
     waitpid(-1,NULL,WNOHANG);
 }
 
-int dowait()
+kint dowait()
 {
     int status;
     wait(&status);
     return status;
 }
 
-int dowaitpid(int pid)
+kint dowaitpid(kint pid)
 {
     int status;
     waitpid(pid,&status,0);
-    return status;
+    return (kint)status;
 }
 #endif
 
 
-int dogetpid()
+kint dogetpid()
 {
     pid_t t = getpid();
-    return (int)t;
+    return (kint)t;
 }
 
 bool do_wcschr(wchar_t c, wchar_t* s) {
   return wcschr(s,c)!=NULL;
 }
 
-int do_wcscspn(wchar_t c, wchar_t* s) {
+kint do_wcscspn(wchar_t c, wchar_t* s) {
   wchar_t buf[2];
   buf[0] = c;
   buf[1] = '\0';
   return wcscspn(s,buf);
 }
 
+kint do_wcsstr(wchar_t* needle, wchar_t* haystack) {
+  wchar_t* hptr = wcsstr(haystack,needle);
+  if (hptr == NULL) {
+    return -1;
+  } else {
+    return hptr-haystack;
+  }
+}
+
+kint offset_wcscspn(wchar_t c, wchar_t* s, kint i) {
+  wchar_t buf[2];
+  buf[0] = c;
+  buf[1] = '\0';
+  return wcscspn(s+i,buf);
+}
+
 /** The following base64 functions are adapted from
  * http://base64.sourceforge.net/b64.c
  * By Bob Trower, Copyright (c) Trantor Standard Systems Inc., 2001
@@ -521,7 +590,7 @@ static const char cd64[]="|$$$}rstuvwxyz{$$$$$$$>?@ABCDEFGHIJKLMNOPQRSTUVW$$$$$$
 **
 ** encode 3 8-bit binary bytes as 4 '6-bit' characters
 */
-void encodeblock( unsigned char in[3], unsigned char out[4], int len )
+void encodeblock( unsigned char in[3], unsigned char out[4], kint len )
 {
     out[0] = cb64[ in[0] >> 2 ];
     out[1] = cb64[ ((in[0] & 0x03) << 4) | ((in[1] & 0xf0) >> 4) ];
@@ -541,13 +610,13 @@ void decodeblock( unsigned char in[4], unsigned char out[3] )
     out[ 2 ] = (unsigned char ) (((in[2] << 6) & 0xc0) | in[3]);
 }
 
-wchar_t* b64enc(wchar_t* wblock, int len)
+wchar_t* b64enc(wchar_t* wblock, kint len)
 {
   return strtowc(b64enc(wctostr(wblock),len));
 }
 
 // since may want to encode binary data
-char* b64enc(char* block, int len) {
+char* b64enc(char* block, kint len) {
     char* outblock = (char*)GC_malloc(sizeof(char)*len*2);
     unsigned char in[3], out[4];
     int i,x,y;
@@ -579,10 +648,10 @@ wchar_t* b64dec(wchar_t* wblock, KayaValue outlen)
 }
 // again, may want to return binary data
 char* b64dec(char* block, KayaValue outlen) {
-    int len = strlen(block);
+    kint len = strlen(block);
     char* outblock = (char*)GC_malloc(sizeof(char)*len);
     unsigned char in[4], out[3];
-    int i,x,y;
+    kint i,x,y;
     x=0; // Input pointer
     y=0; // Output pointer
     while(x<len) {
@@ -640,8 +709,8 @@ int partition(VMState* vm, KayaArray xs, int l, int r, int pi,
     KayaValue xpi = xs->lookup(pi);
     Value pivotval_stack(NULL,xpi->getFunTable()); KayaValue pivotval=&pivotval_stack;
     Value tmp_stack(NULL,KVT_INT); KayaValue tmp=&tmp_stack;
-//    static KayaValue pivotval = new Value(NULL,inttable);
-//    static KayaValue tmp = new Value(NULL,inttable);
+//    static KayaValue pivotval = MKVAL(NULL,inttable);
+//    static KayaValue tmp = MKVAL(NULL,inttable);
     pivotval->setPtr(xpi);
 //    swap(xpi,xs->lookup(r));
     KayaValue xsr = xs->lookup(r);
@@ -658,7 +727,7 @@ int partition(VMState* vm, KayaArray xs, int l, int r, int pi,
 
     int si = l;
     int res;
-
+    Value rv(NULL,KVT_NULL);
     for(int i=l; i<r; ++i) {
 	xsi = xs->lookup(i);
 	xssi = xs->lookup(si);
@@ -672,6 +741,7 @@ int partition(VMState* vm, KayaArray xs, int l, int r, int pi,
 			 pivotval->getString()->getVal());
 	    break;
 	default:
+	    vm->push(&rv);
 	    vm->push(pivotval);
 	    vm->push(xsi);
 	    sortfn->run(vm);
@@ -703,7 +773,7 @@ int partition(VMState* vm, KayaArray xs, int l, int r, int pi,
 }
 
 
-int funtable_compare(void* vmptr,KayaValue x,KayaValue y) 
+kint funtable_compare(void* vmptr,KayaValue x,KayaValue y) 
 {
 #ifndef NOCHECK
   // NULL pointer for vmptr means something else is confident x and y exist
@@ -712,7 +782,7 @@ int funtable_compare(void* vmptr,KayaValue x,KayaValue y)
 	vm->kaya_rtsError(INVALID_VALUE);
     }
 #endif
-    int r = 0;
+    kint r = 0;
     FUNTABLE_OP(cmp,x,y)
 	return r;
 }
@@ -760,8 +830,8 @@ bool funtable_equal_aux(KayaValue x, KayaValue y, map<Value*, Value*>& done)
     case KVT_ARRAY: 
 	r = arraytable_eq(x,y,done); 
 	break; 
-    case KVT_UNION: 
-	r = uniontable_eq(x,y,done); 
+    case KVT_CUNION: 
+	r = cuniontable_eq(x,y,done); 
 	break; 
     case KVT_EXCEPTION: 
 	r = exceptiontable_eq(x,y,done); 
@@ -772,46 +842,17 @@ bool funtable_equal_aux(KayaValue x, KayaValue y, map<Value*, Value*>& done)
     case KVT_NULL: 
 	r = inttable_eq(x,y,done);
 	break; // can't happen
+    case KVT_UNION: 
+    default:
+	r = uniontable_eq(x,y,done); 
+	break; 
     } 
     return r;
 }
 
-
-int funtable_hash(KayaValue x)
+kint funtable_memusage(KayaValue x)
 {
-    Value* r = new Value(NULL,KVT_NULL);
-    switch (x->getFunTable()) { 
-    case KVT_INT: 
-	r = inttable_hash(x); 
-	break; 
-    case KVT_STRING: 
-	r = stringtable_hash(x); 
-	break; 
-    case KVT_FUNC: 
-	r = fntable_hash(x); 
-	break; 
-    case KVT_ARRAY: 
-	r = arraytable_hash(x); 
-	break; 
-    case KVT_UNION: 
-	r = uniontable_hash(x); 
-	break; 
-    case KVT_EXCEPTION: 
-	r = exceptiontable_hash(x); 
-	break; 
-    case KVT_REAL: 
-	r = realtable_hash(x); 
-	break; 
-    case KVT_NULL: 
-	r = inttable_hash(x);
-	break; // can't happen
-    } 
-    return r->getInt();
-}
-
-int funtable_memusage(KayaValue x)
-{
-    int r = 0;
+    kint r = 0;
     switch (x->getFunTable()) { 
     case KVT_INT: 
 	r = inttable_memusage(x); 
@@ -825,8 +866,8 @@ int funtable_memusage(KayaValue x)
     case KVT_ARRAY: 
 	r = arraytable_memusage(x); 
 	break; 
-    case KVT_UNION: 
-	r = uniontable_memusage(x); 
+    case KVT_CUNION: 
+	r = cuniontable_memusage(x); 
 	break; 
     case KVT_EXCEPTION: 
 	r = exceptiontable_memusage(x); 
@@ -837,11 +878,15 @@ int funtable_memusage(KayaValue x)
     case KVT_NULL: 
 	r = inttable_memusage(x);
 	break;
+    case KVT_UNION: 
+    default:
+	r = uniontable_memusage(x); 
+	break; 
     } 
     return r;
 }
 
-wchar_t* funtable_marshal(void* vmptr,KayaValue x, int i)
+wchar_t* funtable_marshal(void* vmptr,KayaValue x, kint i)
 {
 #ifndef NOCHECK
     if (vmptr != NULL && (x == NULL || x->getType() == KVT_NULL)) {
@@ -850,45 +895,47 @@ wchar_t* funtable_marshal(void* vmptr,KayaValue x, int i)
     }
 #endif
     vector<KayaValue> done;
-    wchar_t* mdata = funtable_marshal_aux(vmptr,done,x,i);
-    wchar_t mid[27]; // can be negative
-    SWPRINTF(mid,27,L"[%d][%d]",i,getFnHash());
-    String* idmdata = new String(mid);
-    idmdata->append(mdata);
+    
+    wchar_t mid[50]; // can be negative
+    SWPRINTF(mid,50,MARSHAL2FORMAT,i,getFnHash());
+    String* idmdata = NEWSTRING(mid);
+    funtable_marshal_aux(vmptr,idmdata,done,x,i);
     return idmdata->getVal();
 }
 
-wchar_t* funtable_marshal_aux(void* vmptr,vector<KayaValue>& done,KayaValue x, int i)
+void funtable_marshal_aux(void* vmptr,String* mdata,vector<KayaValue>& done,KayaValue x, kint i)
 {
     VMState* vm = (VMState*)vmptr;
-    Value* r = new Value(NULL,KVT_NULL);
     switch (x->getFunTable()) { 
     case KVT_INT: 
-      r = inttable_marshal(vm,done,x,i);
+      inttable_marshal(vm,mdata,done,x,i);
       break;
     case KVT_STRING: 
-      r = stringtable_marshal(vm,done,x,i);
+      stringtable_marshal(vm,mdata,done,x,i);
       break;
     case KVT_FUNC: 
-      r = fntable_marshal(vm,done,x,i);
+      fntable_marshal(vm,mdata,done,x,i);
       break;
     case KVT_ARRAY: 
-      r = arraytable_marshal(vm,done,x,i); 
+      arraytable_marshal(vm,mdata,done,x,i); 
       break;
-    case KVT_UNION: 
-      r = uniontable_marshal(vm,done,x,i);
+    case KVT_CUNION: 
+      cuniontable_marshal(vm,mdata,done,x,i);
       break;
     case KVT_EXCEPTION: 
-      r = exceptiontable_marshal(vm,done,x,i);
+      exceptiontable_marshal(vm,mdata,done,x,i);
       break;
     case KVT_REAL: 
-      r = realtable_marshal(vm,done,x,i); 
+      realtable_marshal(vm,mdata,done,x,i); 
       break;
     case KVT_NULL:
-      r = inttable_marshal(vm,done,x,i);
+      inttable_marshal(vm,mdata,done,x,i);
       break; //can't happen
+    case KVT_UNION: 
+    default:
+      uniontable_marshal(vm,mdata,done,x,i);
+      break;
     } 
-    return r->getString()->getVal();
 }
 
 Value* funtable_reflect(void* vmptr,KayaValue x)
@@ -907,7 +954,7 @@ Value* funtable_reflect_aux(void* vmptr,
 			    map<KayaValue, KayaValue>& done,KayaValue x)
 {
     VMState* vm = (VMState*)vmptr;
-    Value* r = new Value(NULL,KVT_NULL);
+    Value* r; // = MKVAL(NULL,KVT_NULL);
     switch (x->getFunTable()) { 
     case KVT_INT: 
       r = inttable_reflect(vm,done,x);
@@ -921,8 +968,8 @@ Value* funtable_reflect_aux(void* vmptr,
     case KVT_ARRAY: 
       r = arraytable_reflect(vm,done,x); 
       break;
-    case KVT_UNION: 
-      r = uniontable_reflect(vm,done,x);
+    case KVT_CUNION: 
+      r = cuniontable_reflect(vm,done,x);
       break;
     case KVT_EXCEPTION: 
       r = exceptiontable_reflect(vm,done,x);
@@ -933,32 +980,36 @@ Value* funtable_reflect_aux(void* vmptr,
     case KVT_NULL:
       r = inttable_reflect(vm,done,x);
       break; //can't happen
+    case KVT_UNION: 
+    default:
+      r = uniontable_reflect(vm,done,x);
+      break;
     } 
     return r;
 }
 
-KayaValue reifyUnion(int tag, int arity)
+KayaValue reifyUnion(kint tag, kint arity)
 {
     return KayaUnion(tag, arity);
 }
 
 void reifyUnionArgs(KayaValue v, KayaArray flds)
 {
-    for(int i=0;i<flds->size();++i) {
+    for(unsigned i=0;i<flds->size();++i) {
 	KayaUnionSetArg(v,i,KayaArrayLookup(flds,i));
     }
 }
 
-KayaValue reifyClosure(int fnid, int arity)
+KayaValue reifyClosure(kint fnid, kint arity)
 {
     Closure* c = new Closure(NULL, getFn(fnid), arity, arity, false);
-    return new Value((void*)c,KVT_FUNC);
+    return MKVAL((void*)c,KVT_FUNC);
 }
 
 void reifyClosureArgs(KayaValue v, KayaArray flds)
 {
     Closure* c = v->getFunc();
-    for(int i=0;i<flds->size();++i) {
+    for(unsigned i=0;i<flds->size();++i) {
 	c->setArg(i,KayaArrayLookup(flds,i));
     }
 }
@@ -989,7 +1040,7 @@ KayaValue funtable_copy(void* vmptr,KayaValue x)
 
 KayaValue funtable_copy_aux(KayaValue x, map<Value*, Value*>& done)
 {
-    KayaValue r = new Value(NULL,KVT_NULL);
+    KayaValue r; // = MKVAL(NULL,KVT_NULL);
     FUNTABLE_OP(copy,x,done)
     return r;
 }
@@ -1016,9 +1067,9 @@ wchar_t* except_msg(KayaValue x)
     return x->getExcept()->err->getVal();
 }
 
-int except_code(KayaValue x)
+kint except_code(KayaValue x)
 {
-    return x->getExcept()->code;
+    return (kint)x->getExcept()->code;
 }
 
 void except_dumpbt(KayaValue x)
@@ -1026,14 +1077,14 @@ void except_dumpbt(KayaValue x)
     x->getExcept()->dumpBacktrace();
 }
 
-int doGetFnID(KayaValue fn)
+kint doGetFnID(KayaValue fn)
 {
 //    if (fn->getType()!=KVT_FUNC) { return -1; }
 //    cout << "Raw: " << fn->getRaw() << endl;
-    return getFnID(fn->getFunc()->getfn());
+    return (kint)getFnID(fn->getFunc()->getfn());
 }
 
-void callFnID(void* vmptr, int id, KayaValue arg)
+void callFnID(void* vmptr, kint id, KayaValue arg)
 {
     VMState* vm = (VMState*)vmptr;
     func f = getFn(id);
@@ -1045,7 +1096,7 @@ void callFnID(void* vmptr, int id, KayaValue arg)
     }
 }
 #ifndef WIN32
-int memusage()
+kint memusage()
 {
 /*    struct rusage usage;
     int x = getrusage(RUSAGE_SELF,&usage);
@@ -1058,21 +1109,21 @@ int memusage()
     return 0;
 }
 #endif
-int isNull(void* p)
+kint isNull(void* p)
 {
     return p==NULL;
 }
 
-int isIdentical(KayaValue x, KayaValue y)
+kint isIdentical(KayaValue x, KayaValue y)
 {
     return (x->getRaw()==y->getRaw());
 }
 
 /* Code to handle do_access() in IO.k 
 // Added by Chris Morris, 18/6/05 */
-int do_access(wchar_t* pathname, KayaValue mode) {
-  int tag = KayaUnionGetTag(mode);
-  int acmode = 0;
+kint do_access(wchar_t* pathname, KayaValue mode) {
+  kint tag = KayaUnionGetTag(mode);
+  kint acmode = 0;
   switch(tag) {
   case 0:
     acmode = F_OK;
@@ -1087,11 +1138,11 @@ int do_access(wchar_t* pathname, KayaValue mode) {
     acmode = R_OK;
     break;
   }
-  return access(wctostr(pathname),acmode);
+  return (kint)access(wctostr(pathname),acmode);
 
 }
 #ifndef WIN32
-int getres(KayaValue lim, bool& ok)
+kint getres(KayaValue lim, bool& ok)
 {
     ok = true;
     switch(KayaUnionGetTag(lim)) {
@@ -1153,7 +1204,7 @@ int getres(KayaValue lim, bool& ok)
     }
 }
 
-int do_setrlimit(KayaValue lim, int soft, int hard)
+kint do_setrlimit(KayaValue lim, kint soft, kint hard)
 {
     struct rlimit r;
     if (soft==-1) {
@@ -1177,7 +1228,7 @@ int do_setrlimit(KayaValue lim, int soft, int hard)
 }
 #endif
 
-int do_unlink(const wchar_t* fn) {
+kint do_unlink(const wchar_t* fn) {
   return unlink(wctostr(fn));
 }
 
@@ -1193,7 +1244,7 @@ wchar_t* dogetcwd() {
   return KSTRING(buf);
 }
 
-int do_mkdir(const wchar_t* fn, int um) {
+kint do_mkdir(const wchar_t* fn, kint um) {
 #ifdef WIN32
 	return mkdir(wctostr(fn));
 #else
@@ -1210,7 +1261,7 @@ void* do_opendir(wchar_t* name)
     return (void*)d;
 }
 
-int do_closedir(void* dir)
+kint do_closedir(void* dir)
 {
     return closedir((DIR*)dir);
 }
@@ -1328,15 +1379,15 @@ KayaValue do_stat(void* vmptr, wchar_t* name)
     return val;
 }
 
-int maxMemUsage(void* vmptr)
+kint maxMemUsage(void* vmptr)
 {
     VMState* vm = (VMState*)vmptr;
-    return vm->maxMemUsage();
+    return (kint)vm->maxMemUsage();
 }
 
-wchar_t* do_urlencode(void* vm, wchar_t* strinwide)
+wchar_t* do_urlencode(void* vm, wchar_t* strinwide, kint len)
 {
-  char* strin = CSTRING(strinwide);
+  char* strin = CNSTRING(strinwide,len);
     VMState* vmptr = (VMState*)vm;
     char* strout = (char*)KayaAlloc(strlen(strin)*3+1);
     if (strout==NULL) {
@@ -1366,9 +1417,9 @@ wchar_t* do_urlencode(void* vm, wchar_t* strinwide)
     return KSTRING(r);
 }
 
-wchar_t* do_urldecode(void* vm, wchar_t* strinwide)
+wchar_t* do_urldecode(void* vm, wchar_t* strinwide, kint len)
 {
-  char* strin = CSTRING(strinwide);
+  char* strin = CNSTRING(strinwide,len);
     VMState* vmptr = (VMState*)vm;
     char* strout = (char*)KayaAlloc(strlen(strin)+1);
     if (strout==NULL) {
@@ -1409,12 +1460,12 @@ wchar_t* do_urldecode(void* vm, wchar_t* strinwide)
     return KSTRING(r);
 }
 
-FILE *do_fopen(const wchar_t* path, const wchar_t* mode) {
-    return fopen(wctostr(path),wctostr(mode));
+void* do_fopen(const wchar_t* path, const wchar_t* mode) {
+    return (void*)fopen(wctostr(path),wctostr(mode));
 }
 
-FILE *do_freopen(const wchar_t* path, const wchar_t* mode, FILE* stream) {
-    return freopen(wctostr(path),wctostr(mode),stream);
+void* do_freopen(const wchar_t* path, const wchar_t* mode, void* stream) {
+    return (void*)freopen(wctostr(path),wctostr(mode),(FILE*)stream);
 }
 
 void do_rename(const wchar_t* oldfile, const wchar_t* newfile) {
@@ -1430,46 +1481,46 @@ wchar_t* do_getenv(const wchar_t* env) {
 }
 
 #ifndef WIN32
-int do_system(const wchar_t* prog) {
+kint do_system(const wchar_t* prog) {
     return system(wctostr(prog));
 }
 
-FILE* do_popen(const wchar_t* prog, int read) {
+void* do_popen(const wchar_t* prog, kint read) {
     if (read) {
-	return popen(wctostr(prog),"r");
+	return (void*)popen(wctostr(prog),"r");
     } else {
-	return popen(wctostr(prog),"w");
+	return (void*)popen(wctostr(prog),"w");
     }
 }
 
-int do_pclose(FILE* stream) {
-    return pclose(stream);
+kint do_pclose(void* stream) {
+    return pclose((FILE*)stream);
 }
 
 #endif
 
-kint binary_length(const wchar_t* str) {
-    char* rawstr = CSTRING(str);
+kint binary_length(const wchar_t* str, kint len) {
+    char* rawstr = CNSTRING(str,len);
     return (kint)strlen(rawstr);
 }
 
-int nextPower(int x)
+kint nextPower(kint x)
 {
-    int next = 1;
+    kint next = 1;
     while(x>next) {
 	next<<=1;
     }
     return next;
 }
 
-int strAlloc(Value* str)
+kint strAlloc(Value* str)
 {
     return str->getString()->space()*sizeof(wchar_t);
 }
 
-Value* createString(int len)
+Value* createString(kint len)
 {
-    return new Value((void*)(new String(len)),KVT_STRING);
+    return MKVAL((void*)(NEWSTRING(len)),KVT_STRING);
 }
 
 void do_GC_enable_incremental() {
@@ -1478,7 +1529,7 @@ void do_GC_enable_incremental() {
 #endif
 }
 
-void do_sleep(int len) {
+void do_sleep(kint len) {
 #ifdef WIN32
 	Sleep(len*1000);
 #else
@@ -1486,7 +1537,7 @@ void do_sleep(int len) {
 #endif
 }
 
-void do_usleep(int len) {
+void do_usleep(kint len) {
 #ifdef WIN32
 	Sleep(len/1000);
 #else
@@ -1494,17 +1545,16 @@ void do_usleep(int len) {
 #endif
 }
 
-int strHash(wchar_t* m)
+kint strHash(wchar_t* m, kint len)
 {
     int hash = 0;
-    int len = wcslen(m);
     for(int i=0;i<len;++i) {
 	hash = 131*hash+m[i];
     }
-    return hash;
+    return (kint)hash;
 }
 
-int rtchecks()
+kint rtchecks()
 {
     return RTCHECKS;
 }
