@@ -71,16 +71,16 @@ void setBlockData(void* block, int offset, int size, KayaArray data)
     }
 }
 
-int writeBlock(FILE* f,void* block, int size)
+int writeBlock(void* f,void* block, int size)
 {
-    return fwrite(block,1,size,f);
+    return fwrite(block,1,size,(FILE*)f);
 }
 
-void* readBlock(FILE* f, KayaValue size)
+void* readBlock(void* f, KayaValue size)
 {
     int isize = KayaGetInt(size);
     void* newblock = (void*)GC_MALLOC_ATOMIC(isize);
-    isize = fread(newblock,1,isize,f);
+    isize = fread(newblock,1,isize,(FILE*)f);
     KayaSetInt(size,isize);
     return newblock;
 }
