@@ -130,7 +130,7 @@ char* decode(void* vmptr,char* str)
     // If the length is much longer than the original string we're decoding,
     // this is clearly nonsense, so return an error.
     if (total>strlen(str)) {
-	vm->kaya_throw("Invalid string to decode",255);
+	vm->kaya_internalError(255);
     }
 
     unsigned char *dec, *junk, *msghash;
@@ -184,7 +184,7 @@ char* decode(void* vmptr,char* str)
 	val->append(hash2);
 	val->append(br);
 	val->append(err);
-	vm->kaya_throw(val->getVal(),255);
+	vm->kaya_rtsError(255);
    }
 
     return (char*)(dec+20);
