@@ -27,8 +27,8 @@ public:
 
 extern "C" {
 
-void* re_compile(void* vm, wchar_t* pattern, kint flagcode);
-void* re_match(void* vm, void* pc, wchar_t* str);
+void* re_compile(void* vm, wchar_t* pattern, kint len, kint flagcode);
+void* re_match(void* vm, void* pc, wchar_t* str, kint len);
 
 Array* getmatches(void* mo);
 bool matched(void* mo);
