@@ -24,8 +24,14 @@ extern "C" {
   void* do_gnutls_init(void* cred);
   int do_gnutls_transport(void* tls, void* conn);
   void do_gnutls_put(void* tls, wchar_t* rawdat);
+  void do_gnutls_putbin(void* tls, void* rawdat, kint len);
+  void do_gnutls_putbyte(void* tls, kint byte);
   wchar_t* do_gnutls_get(void* tls, void* vmptr);
+  void* do_gnutls_getbin(void* tls, void* vmptr, KValue len);
+  kint do_gnutls_getbyte(void* tls, void* vmptr);
   void do_gnutls_close(void* tls, void* cred);
+  void do_addcert(void* vmptr, void* credptr, wchar_t* fn);
+  void do_verifycert(void* vmptr, void* sessptr, wchar_t* host);
 }
 
 #endif
